@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -31,7 +31,7 @@
  #define WIN_MAX_PATH (4*MAX_PATH)
 #endif
 
-#if 0
+#if 0 
  #define Verbose g_printf
 #else
  #define Verbose(format, ...)
@@ -165,6 +165,10 @@ static void get_base_dirs()
 {  
 #ifdef SYS_MINGW
    char *appdata;
+   char *homedir;
+#endif
+#ifdef SYS_DARWIN
+   char *resdir;
 #endif
 
    /*** Unless completely disabled through a configure option, the
@@ -188,9 +192,12 @@ static void get_base_dirs()
    /*** Otherwise try the installation directory. 
 	On Unices this is a hardcoded directory.
 	Windows has binary distributions with no prior known installation place,
-	but luckily it provides a way for figuring out that location. */
+	but luckily it provides a way for figuring out that location. 
+        Darwin is like Unix when installed via make install;
+        but tricky when app bundles are used. In that case we
+        use GTK_PATH to find out our location. */
 
-#ifndef SYS_MINGW
+#if defined(SYS_LINUX) || defined(SYS_FREEBSD) || defined(SYS_NETBSD) || defined(SYS_UNKNOWN) || defined(SYS_SOLARIS)
    if(DirStat(BINDIR))
      Closure->binDir = g_strdup(BINDIR);
 
@@ -199,6 +206,29 @@ static void get_base_dirs()
    Verbose("Using hardcoded BINDIR = %s, DOCDIR = %s\n", BINDIR, DOCDIR);
 #endif
 
+#ifdef SYS_DARWIN
+   resdir = (char*)g_getenv("GTK_PATH");
+   if(resdir)  /* looks like we are inside an app bundle */
+   {  if(DirStat(resdir))
+         Closure->binDir = g_strdup(resdir);
+
+      Closure->docDir = g_strdup_printf("%s/doc", resdir);
+      if(!DirStat(Closure->docDir))
+      {  g_free(Closure->docDir);
+	 Closure->docDir = NULL;
+      }
+   }
+   else  /* maybe a normal install */
+   {
+      if(DirStat(BINDIR))
+         Closure->binDir = g_strdup(BINDIR);
+
+      if(DirStat(DOCDIR))
+         Closure->docDir = g_strdup(DOCDIR);
+   }
+   Verbose("Using BINDIR = %s, DOCDIR = %s\n", BINDIR, DOCDIR);
+#endif
+
 #ifdef SYS_MINGW
    Closure->binDir = get_exe_path();
 
@@ -214,22 +244,34 @@ static void get_base_dirs()
  #ifdef WITH_EMBEDDED_SRC_PATH_YES
 find_dotfile:
  #endif /* WITH_EMBEDDED_SRC_PATH_YES */
-
-   Closure->dotFile = g_strdup_printf("%s/.dvdisaster", g_getenv("HOME"));
+   
+   Closure->homeDir = g_strdup(g_getenv("HOME"));
+   Closure->dotFile = g_strdup_printf("%s/.dvdisaster", Closure->homeDir);
 #endif
 
 #ifdef SYS_MINGW
    /* For Windows the user's application directory in the roaming
-      profile is preferred; 
+      profile is preferred for the dotfile; 
       if it does not exist we use the installation directory.  */
 
    appdata = get_special_folder(CSIDL_APPDATA);
+   homedir = get_special_folder(CSIDL_PERSONAL);
    Verbose("Windows specific paths:\n"
-	   "- CSIDL_APPDATA: %s\n",
-	   appdata ? appdata : "NULL");
+	   "- CSIDL_APPDATA: %s\n"
+	   "- CSIDL_PERSONAL: %s\n",
+	   appdata ? appdata : "NULL",
+	   homedir ? homedir : "NULL");
+
+   if(homedir && DirStat(homedir))
+   {
+      Closure->homeDir = g_strdup(homedir);
+      Verbose("- homedir path : %s\n", Closure->homeDir);
+
+      g_free(homedir);
+   }
 
    if(appdata)
-   {  Closure->appData = g_strdup_printf("%s\\dvdisaster", appdata);
+   {  Closure->appData  = g_strdup_printf("%s\\dvdisaster", appdata);
 
       if(DirStat(appdata)) /* CSIDL_APPDATA present? */
       { 
@@ -257,20 +299,25 @@ find_dotfile:
       g_free(appdata);
    }
 
-   /* Fallback: Expect .dvdisaster file in binDir */
+   /* Fallbacks: Expect .dvdisaster file in binDir;
+                 propose C:\Windows\Temp as working dir */
 
    if(!Closure->dotFile)
       Closure->dotFile = g_strdup_printf("%s\\.dvdisaster", Closure->binDir);
+   if(!Closure->homeDir)
+      Closure->homeDir = g_strdup("C:\\Windows\\Temp");
 #endif
 
    Verbose("\nUsing file locations:\n"
+	   "- Homedir: %s\n"
 	   "- Bin dir: %s\n"
 	   "- Doc dir: %s\n"
 	   "- AppData: %s\n"
 	   "- dotfile: %s\n\n",
+	   Closure->homeDir,
 	   Closure->binDir,
 	   Closure->docDir,
-	   Closure->appData,
+	   Closure->appData ? Closure->appData : "Null",
 	   Closure->dotFile);   
 }
 
@@ -445,10 +492,7 @@ void ReadDotfile()
       if(!strcmp(symbol, "dvd-size1"))       { Closure->dvdSize1 = Closure->savedDVDSize1 = atoll(value); continue; }
       if(!strcmp(symbol, "dvd-size2"))       { Closure->dvdSize2 = Closure->savedDVDSize2 = atoll(value); continue; }
       if(!strcmp(symbol, "eject"))           { Closure->eject  = atoi(value); continue; }
-      if(!strcmp(symbol, "fill-unreadable")) { Closure->fillUnreadable = atoi(value); 
-	                                       PrepareDeadSector();
-	                                       continue; 
-                                             }
+      if(!strcmp(symbol, "fill-unreadable")) { Closure->fillUnreadable = atoi(value); continue; }
       if(!strcmp(symbol, "ignore-fatal-sense")) { Closure->ignoreFatalSense  = atoi(value); continue; }
       if(!strcmp(symbol, "internal-attempts"))  { Closure->internalAttempts = atoi(value); continue; }
       if(!strcmp(symbol, "jump"))            { Closure->sectorSkip  = atoi(value); continue; }
@@ -460,6 +504,7 @@ void ReadDotfile()
 	                                       Closure->methodName = g_strdup(value); continue; }
       if(!strcmp(symbol, "max-read-attempts"))   { Closure->maxReadAttempts = atoi(value); continue; }
       if(!strcmp(symbol, "min-read-attempts"))   { Closure->minReadAttempts = atoi(value); continue; }
+      if(!strcmp(symbol, "missing-sector-marker"))  { Closure->dsmVersion  = atoi(value); continue; }
       if(!strcmp(symbol, "query-size"))      { Closure->querySize  = atoi(value); continue; }
       if(!strcmp(symbol, "raw-mode"))        { Closure->rawMode = atoi(value); continue; }
       if(!strcmp(symbol, "read-and-create")) { Closure->readAndCreate = atoi(value); continue; }
@@ -528,12 +573,12 @@ static void update_dotfile()
    g_fprintf(dotfile, "last-device:       %s\n", Closure->device);
    g_fprintf(dotfile, "last-image:        %s\n", Closure->imageName);
    g_fprintf(dotfile, "last-ecc:          %s\n", Closure->eccName);
-   g_fprintf(dotfile, "bd-size1:          %lld\n", (long long int)Closure->bdSize1);
-   g_fprintf(dotfile, "bd-size2:          %lld\n", (long long int)Closure->bdSize2);
    g_fprintf(dotfile, "browser:           %s\n\n", Closure->browser);
 
    g_fprintf(dotfile, "adaptive-read:     %d\n", Closure->adaptiveRead);
    g_fprintf(dotfile, "auto-suffix:       %d\n", Closure->autoSuffix);
+   g_fprintf(dotfile, "bd-size1:          %lld\n", (long long int)Closure->bdSize1);
+   g_fprintf(dotfile, "bd-size2:          %lld\n", (long long int)Closure->bdSize2);
    g_fprintf(dotfile, "cache-size:        %d\n", Closure->cacheMB);
    g_fprintf(dotfile, "cd-size:           %lld\n", (long long int)Closure->cdSize);
    g_fprintf(dotfile, "codec-threads:     %d\n", Closure->codecThreads);
@@ -555,6 +600,7 @@ static void update_dotfile()
    g_fprintf(dotfile, "method-name:       %s\n", Closure->methodName);
    g_fprintf(dotfile, "max-read-attempts: %d\n", Closure->maxReadAttempts);
    g_fprintf(dotfile, "min-read-attempts: %d\n", Closure->minReadAttempts);
+   g_fprintf(dotfile, "missing-sector-marker: %d\n", Closure->dsmVersion);
    g_fprintf(dotfile, "query-size:        %d\n", Closure->querySize);
    g_fprintf(dotfile, "raw-mode:          %d\n", Closure->rawMode);
    g_fprintf(dotfile, "read-and-create:   %d\n", Closure->readAndCreate);
@@ -638,17 +684,18 @@ void InitClosure()
    printf("Version %s; %d; Flags %d\n", Closure->cookedVersion, Closure->version, Closure->releaseFlags);
 #endif
 
+   /* Get home and system directories */
+
+   get_base_dirs();
+
    /* Fill in other closure defaults */
 
    Closure->deviceNames = g_ptr_array_new();
    Closure->deviceNodes = g_ptr_array_new();
-   Closure->imageName   = g_strdup("medium.iso");
-   Closure->eccName     = g_strdup("medium.ecc");
    Closure->browser     = g_strdup("xdg-open");
    Closure->methodList  = g_ptr_array_new();
    Closure->methodName  = g_strdup("RS01");
-   Closure->dDumpDir    = g_strdup("/tmp");
-   Closure->dDumpPrefix = g_strdup("sector-");
+   Closure->dDumpDir    = g_strdup(Closure->homeDir);
    Closure->cacheMB     = 32;
    Closure->codecThreads = 1;
    Closure->minReadAttempts = 1;
@@ -670,6 +717,7 @@ void InitClosure()
    Closure->bdSize2  = Closure->savedBDSize2  = BD_DL_SIZE;
 
    Closure->logString = g_string_sized_new(1024);
+   Closure->logLock   = g_mutex_new();
 
    Closure->background = g_malloc0(sizeof(GdkColor));
    Closure->foreground = g_malloc0(sizeof(GdkColor));
@@ -691,7 +739,6 @@ void InitClosure()
 
    memset(Closure->bs, '\b', 255);
 
-   get_base_dirs();
    DefaultLogFile();
 
 #ifdef SYS_MINGW
@@ -699,6 +746,48 @@ void InitClosure()
 #endif
 }
 
+/*
+ * Add some localized file name defaults.
+ * Can't do this in InitClosure() as the locale has not been
+ * initialized when it is being called. 
+ */
+
+void LocalizedFileDefaults()
+{  
+#ifdef SYS_DARWIN
+   /* Darwin uses / as the current working directory when invoked
+      from an app bundle. Not good. */
+   char *buf = getcwd(NULL,0);
+   
+   Closure->imageName = NULL;
+
+   if(buf && !strcmp(buf,"/"))
+   {  free(buf);
+      buf = getenv("HOME");
+      if(buf)
+      {
+         Closure->imageName   = g_strdup_printf("%s/%s",buf,_("medium.iso"));
+         Closure->eccName     = g_strdup_printf("%s/%s",buf,_("medium.ecc"));
+         Closure->dDumpPrefix = g_strdup_printf("%s/%s",buf,_("sector-"));
+      }
+   }
+
+   if(!Closure->imageName)
+   {
+     Closure->imageName   = g_strdup(_("medium.iso"));
+     Closure->eccName     = g_strdup(_("medium.ecc"));
+     Closure->dDumpPrefix = g_strdup(_("sector-"));
+   }
+
+#else
+   /* On all other OS, storing the files in the cwd is a sane default. */
+   Closure->imageName   = g_strdup(_("medium.iso"));
+   Closure->eccName     = g_strdup(_("medium.ecc"));
+   Closure->dDumpPrefix = g_strdup(_("sector-"));
+#endif
+}
+
+
 /*
  * Clear the CRC cache
  */
@@ -753,13 +842,13 @@ void FreeClosure()
    cond_free_ptr_array(Closure->methodList);
 
    cond_free(Closure->methodName);
+   cond_free(Closure->homeDir);
    cond_free(Closure->dotFile);
    cond_free(Closure->logFile);
    cond_free(Closure->binDir);
    cond_free(Closure->docDir);
    cond_free(Closure->appData);
    cond_free(Closure->browser);
-   cond_free(Closure->deadSector);
    cond_free(Closure->errorTitle);
    cond_free(Closure->dDumpDir);
    cond_free(Closure->dDumpPrefix);
@@ -771,7 +860,10 @@ void FreeClosure()
       FreeRawEditorContext(Closure->rawEditorContext);
 
    if(Closure->logString)
-     g_string_free(Closure->logString, TRUE);
+      g_string_free(Closure->logString, TRUE);
+
+   if(Closure->logLock)
+      g_mutex_free(Closure->logLock);
 
    if(Closure->drawGC)
      g_object_unref(Closure->drawGC);
