@@ -1,6 +1,6 @@
 <?php
 # dvdisaster: German homepage translation
-# Copyright (C) 2004-2008 Carsten Gnörlich
+# Copyright (C) 2004-2009 Carsten Gnörlich
 #
 # UTF-8 trigger: äöüß 
 #
@@ -28,25 +28,85 @@ aus der folgenden Liste herunter. Die Pakete können mit
 einer <a href="download20.php">digitalen Unterschrift</a> auf 
 ihren Ursprungszustand überprüft werden.<p>
 
-Um die Binärversion  <a href="download30.php">für Windows zu installieren</a>, 
-rufen Sie das Programm nach dem Herunterladen auf und folgen dem Dialog. <p>
+<ul>
+<li>Die Quellkode-Version enthält eine Datei <tt>INSTALL</tt> mit weiteren
+Informationen zum Übersetzen des Programmes.</li>
+<li>Für Mac OS X laden Sie bitte das ZIP-Archiv herunter und packen es an einer
+beliebigen Stelle aus. Bitte beachten Sie die 
+<a href="download30.php#mac">speziellen Hinweise für Mac OS X</a>.</li>
+<li>Um die Binärversion <a href="download30.php#win">für Windows zu installieren</a>, 
+rufen Sie das Programm nach dem Herunterladen auf und folgen dem Dialog.</li>
+</ul> 
 
 <!---
-<b>Entwickler-Versionen</b> - neu und experimentell für erfahrene Benutzer!<p> 
-<b>Stabile Versionen</b> - zum Einstieg empfohlen<p> 
---->
-
 <b>Alpha-/Entwickler-Versionen</b> - neu und experimentell für erfahrene Benutzer!<p> 
 
 Sie sind eingeladen, die nächste dvdisaster-Version auszuprobieren, 
 aber beachten Sie bitte, daß diese Version noch Fehler und 
 Kompatibilitätsprobleme enthalten kann. Die neueste experimentelle Version 
-ist <a href="download40.php"><?php echo $cooked_version ?></a>.
+ist <a href="download40.php"><?php echo $cooked_version ?></a>. 
+--->
 
 <pre> </pre>
 
 <b>Stabile Version</b> - zum Einstieg empfohlen<p> 
 
+<table class="download" cellpadding="0" cellspacing="5">
+<tr><td><b>dvdisaster-0.72</b></td><td align="right">08-Aug-2009</td></tr>
+<tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
+<tr><td colspan="2">
+  <table>
+    <tr><td align="right">&nbsp;&nbsp;Quellkode für alle Betriebssysteme:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.tar.bz2?download">dvdisaster-0.72.1.tar.bz2</a></td></tr>
+    <tr><td align="right">Digitale Unterschrift:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.tar.bz2.gpg?download">dvdisaster-0.72.1.tar.bz2.gpg</a></td></tr>
+    <tr><td align="right">Binärversion für Mac OS X 10.5 / x86:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.app.zip?download">dvdisaster-0.72.1.app.zip</a>&nbsp;--&nbsp;bitte erst den <a href="download30.php#mac">Hinweis</a> lesen</td></tr>
+    <tr><td align="right">Digitale Unterschrift:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.app.zip.gpg?download">dvdisaster-0.72.1.app.zip.gpg</a></td></tr>
+    <tr><td align="right">Binärversion für Windows:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1-setup.exe?download">dvdisaster-0.72.1-setup.exe</a></td></tr>
+    <tr><td align="right">Digitale Unterschrift:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1-setup.exe.gpg?download">dvdisaster-0.72.1-setup.exe.gpg</a></td></tr>
+  </table>
+</td></tr>
+<tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
+<tr><td colspan="2">
+Grundlegende Neuerungen in dieser Version:<p>
+<ul>
+<li>Unterstützung für <a href="qa10.php#media">Blu-Ray-Datenträger</a></li>
+<li>"Raw"-Lesen und C2-Überprüfungen für CD-Datenträger</li>
+<li>Einstellbare Anzahl der Leseversuche</li>
+<li>Erstes "natives" Mac OS X-Paket </li>
+<li>NetBSD-Port von Sergey Svishchev</li>
+<li>Verbesserte Typerkennung für eingelegte Datenträger</li>
+<li>Informationsfenster für eingelegten Datenträger</li>
+<li>Überarbeiteter und erweiterter Dialog für Programmeinstellungen</li>
+<li>Neu gestaltete und erweiterte Dokumentation</li>
+<li>Russische Übersetzungen von Igor Gorbounov</li>
+<li>... und unzählige weitere kleine Sachen.</li>
+</ul>
+
+<b>Patches</b> (kleine Änderungen nach Version 0.72; die obigen Dateien wurden erneuert):<p>
+
+<b>0.72 pl1</b> Pablo Almeida hat die Bildschirmtexte ins Portugiesische übersetzt.
+Es wurde ein Workaround eingebaut um das Einfrieren von Windows XP bei bestimmten Kombinationen
+von CD-RW-Rohlingen und Laufwerken zu verhindern. (08-Aug-2009)<p>
+
+<b>0.72</b> Dies ist die erste stabile Version des 0.72er-Zweiges.
+Igor Gorbounov hat die russische online-Dokumentation vervollständigt
+und es wurden noch einige kleine Fehler aus dem ersten 
+Veröffentlichungskandidaten behoben.<p>
+Unter neueren Windows-Versionen wird bei bestimmten Spracheinstellungen
+nicht die gewünschte Bildschirmsprache dargestellt. Die Behebung
+dieses Problems ist ziemlich komplex und wird erst mit Version 0.73
+erfolgen. (04-Jul-2009)<p>
+ 
+<b>0.72-rc1</b> Erster Kandidat für die stabile Version. (11-Apr-2009)
+</td></tr></table><p>
+
+<b>Vorangegangene Version</b> - eine Aktualisierung auf Version 0.72 wird empfohlen.<p> 
+
 <table class="download" cellpadding="0" cellspacing="5">
 <tr><td><b>dvdisaster-0.70</b></td><td align="right">04-Mär-2008</td></tr>
 <tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
@@ -121,25 +181,6 @@ betrachtet werden. Einige interessante Dateien darin sind:
 
 <pre> </pre>
 
-<b>Vorangegangene Version</b> - eine Aktualisierung auf Version 0.70.4 wird empfohlen.<p> 
-
-<table class="download" cellpadding="0" cellspacing="5">
-<tr><td><b>dvdisaster-0.66</b></td><td align="right">25-Mär-2006</td></tr>
-<tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
-<tr><td colspan="2">
-  <table>
-    <tr><td align="right">&nbsp;&nbsp;Quellkode für alle Betriebssysteme:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.tar.bz2?download">dvdisaster-0.66.tar.bz2</a></td></tr>
-    <tr><td align="right">Digitale Unterschrift:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.tar.bz2.gpg?download">dvdisaster-0.66.tar.bz2.gpg</a></td></tr>
-    <tr><td align="right">Binärversion für Windows:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.3-setup.exe?download">dvdisaster-0.66.3-setup.exe</a></td></tr>
-    <tr><td align="right">Digitale Unterschrift:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.3-setup.exe.gpg?download">dvdisaster-0.66.3-setup.exe.gpg</a></td></tr>
-  </table>
-</td></tr>
-</table><p>
-
 <?php
 footnote("*","gpl3","dvdisaster wird zu einem späteren Zeitpunkt zur GNU General Public License Version 3 wechseln.");
 ?>
