@@ -1,6 +1,6 @@
 <?php
 # dvdisaster: German homepage translation
-# Copyright (C) 2004-2008 Carsten Gnörlich
+# Copyright (C) 2004-2009 Carsten Gnörlich
 #
 # UTF-8 trigger: äöüß 
 #
@@ -31,6 +31,7 @@ dvdisaster beinhaltet Bildschirmtexte in den folgenden Sprachen:<p>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Deutsch</td><td>--</td><td>vollständig</td></tr>
 <tr><td></td><td>Englisch</td><td>--</td><td>vollständig</td></tr>
 <tr><td></td><td>Italienisch</td><td>--</td><td>bis Version 0.65</td></tr>
+<tr><td></td><td>Portugiesisch</td><td>--</td><td>vollständig</td></tr>
 <tr><td></td><td>Russisch</td><td>--</td><td>vollständig</td></tr>
 <tr><td></td><td>Schwedisch</td><td>--</td><td>vollständig</td></tr>
 <tr><td></td><td>Tschechisch</td><td>--</td><td>bis Version 0.66</td></tr>
@@ -58,7 +59,8 @@ fehlt eventuell noch: <p>
 
 <b><a name="media">2.2 Welche Datenträger-Typen werden unterstützt?</a></b><p>
 
-dvdisaster unterstützt einmal oder mehrmals beschreibbare CD- und DVD-Datenträger.
+dvdisaster unterstützt einmal oder mehrmals beschreibbare 
+CD-, DVD- und BD-Datenträger.
 Datenträger mit Mehrfachsitzungen (engl.: multisession)
 oder einem Kopierschutz können <i>nicht</i> verwendet werden.<p>
 
@@ -101,8 +103,16 @@ Abhilfe: Abbildgröße aus dem ISO/UDF- oder ECC/RS02-Dateisystem ermitteln.
 wie oben beschrieben möglich.</li>
 </ul>
 
+<b>BD-R, BD-RW</b><p>
+
+<ul>
+<li>Momentan sind keine Einschränkungen bekannt, aber mit
+zweischichtigen Typen (die 50GB-Versionen) liegen noch kaum
+Erfahrungen vor.</li>
+</ul>
+
 <b>Nicht verwendbare Typen</b> (kein Einlesen des Abbildes möglich):<p> 
-DVD-ROM (gepreßte DVDs) sowie Audio-CD und Video-CD.
+BD-ROM (gepreßte BDs), DVD-ROM (gepreßte DVDs) sowie Audio-CD und Video-CD.
 
 <div align=right><a href="#top">&uarr;</a></div><p>
 
