@@ -1,6 +1,6 @@
 <?php
 # dvdisaster: English homepage translation
-# Copyright (C) 2004-2008 Carsten Gnörlich
+# Copyright (C) 2004-2009 Carsten Gnörlich
 #
 # UTF-8 trigger: äöüß 
 #
@@ -27,27 +27,82 @@ Please download either the source code or a binary version from the list below.
 A <a href="download20.php">digital signature</a> is provided for verification that the
 packets are in their original state.<p>
 
-To <a href="download30.php">install</a> the binary version execute 
-the downloaded program and follow the dialog.
-The source code version contains a file <tt>INSTALL</tt> with further instructions.
-
-<pre> </pre>
+<ul>
+<li>The source code version contains a file <tt>INSTALL</tt> with further instructions for compiling it.</li>
+<li>For Mac OS X a ZIP archive is provided which is installed by unpacking it
+into an arbitrary place. Please note the 
+<a href="download30.php#mac">special hints for Mac OS X</a>.</li>
+<li>To <a href="download30.php#win">install</a> the binary version for Windows
+please execute the downloaded program and follow the dialog.</li>
+</ul>
 
 <!---
-<b>Developer versions</b> - new and experimental for experienced users!<p>
-<b>Current version</b><p>
--->
-
 <b>Alpha (unstable) versions</b> - new and experimental for experienced users!<p>
 
 Testers are welcome for the upcoming dvdisaster release, but should be aware of remaining
 bugs and incompatibilities.
 The current unstable release is 
 <a href="download40.php"><?php echo $cooked_version?></a>.
+-->
+
 <pre> </pre>
 
 <b>Stable version</b> - recommended for getting started.<p>
 
+<table class="download" cellpadding="0" cellspacing="5">
+<tr><td><b>dvdisaster-0.72</b></td><td align="right">08-Aug-2009</td></tr>
+<tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
+<tr><td colspan="2">
+  <table>
+    <tr><td align="right">&nbsp;&nbsp;Source code for all operating systems:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.tar.bz2?download">dvdisaster-0.72.1.tar.bz2</a></td></tr>
+    <tr><td align="right">Digital signature:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.tar.bz2.gpg?download">dvdisaster-0.72.1.tar.bz2.gpg</a></td></tr>
+    <tr><td align="right">Binary for Mac OS X 10.5 / x86:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.app.zip?download">dvdisaster-0.72.1.app.zip</a>&nbsp;--&nbsp;please read these <a href="download30.php#mac">hints</a> first</td></tr>
+    <tr><td align="right">Digital signature:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1.app.zip.gpg?download">dvdisaster-0.72.1.app.zip.gpg</a></td></tr>
+    <tr><td align="right">Binary for Windows:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1-setup.exe?download">dvdisaster-0.72.1-setup.exe</a></td></tr>
+    <tr><td align="right">Digital signature:&nbsp;</td>
+        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.72.1-setup.exe.gpg?download">dvdisaster-0.72.1-setup.exe.gpg</a></td></tr>
+  </table>
+</td></tr>
+<tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
+<tr><td colspan="2">
+Most important changes in this version:<p>
+<ul>
+<li>Support for <a href="qa10.php#media">Blu-Ray media</a></li>
+<li>"Raw" reading and C2 scans for CD media</li>
+<li>Selectable number of reading attempts</li>
+<li>First native Mac OS X application bundle</li>
+<li>NetBSD-Port by Sergey Svishchev</li>
+<li>Improved media type recognition</li>
+<li>Info window describing inserted medium</li>
+<li>Improved and enhanced preferences dialog</li>
+<li>Reworked and enhanced documentation</li>
+<li>Russian translations by Igor Gorbounov</li>
+<li>... and many more small changes and fixes.</li>
+</ul>
+
+<b>Patches</b> (small changes after version 0.72; files above have been updated):<p>
+
+<b>0.72 pl1</b> Pablo Almeida provided Portuguese translations of the screen texts.
+Added workaround to avoid Win XP freezing on certain CD-RW/drive pairs.(08-Aug-2009)<p>
+
+<b>0.72</b> This is the first stable version of the 0.72 branch.
+Igor Gorbounov has completed the russian online documentation.
+Some minor bugs from the first release candidate have been removed.<p>
+Newer Windows versions may not display the proper screen language
+under certain language settings. This problem is rather complex
+and will be addressed in the upcoming 0.73.x releases. (04-Jul-2009)
+<p>
+ 
+<b>0.72-rc1</b> First release candidate. (11-Apr-2009)
+</td></tr></table><p>
+
+<b>Previous release</b> - upgrading to version 0.72 is recommended.<p>
+
 <table class="download" cellpadding="0" cellspacing="5">
 <tr><td><b>dvdisaster-0.70</b></td><td align="right">04-Mar-2008</td></tr>
 <tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
@@ -118,28 +173,6 @@ Some file of particular interest are:
 <li><a href="http://dvdisaster.cvs.sourceforge.net/dvdisaster/dvdisaster/README?view=markup">README</a> - a road map of the source code archive.</li>
 </ul>
 
-<b>Previous release</b> - upgrading to version 0.70.4 is recommended.<p>
-
-<table class="download" cellpadding="0" cellspacing="5">
-<tr><td><b>dvdisaster-0.66</b></td><td align="right">25-Mar-2006</td></tr>
-<tr bgcolor="#000000"><td colspan="2"><img width=1 height=1 alt=""></td></tr>
-<tr><td colspan="2">
-  <table>
-    <tr><td align="right">&nbsp;&nbsp;Source code for all operating systems:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.tar.bz2?download">dvdisaster-0.66.tar.bz2</a></td></tr>
-    <tr><td align="right">Digital signature:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.tar.bz2.gpg?download">dvdisaster-0.66.tar.bz2.gpg</a></td></tr>
-    <tr><td align="right">Binary for Windows:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.3-setup.exe?download">dvdisaster-0.66.3-setup.exe</a></td></tr>
-    <tr><td align="right">Digital signature:&nbsp;</td>
-        <td><a href="http://prdownloads.sourceforge.net/dvdisaster/dvdisaster-0.66.3-setup.exe.gpg?download">dvdisaster-0.66.3-setup.exe.gpg</a></td></tr>
-  </table>
-</td></tr>
-</table><p>
-<?php
-footnote("*","gpl3","dvdisaster will switch over to the GNU General Public License version 3 in a later release.");
-?>
-
 <!--- do not change below --->
 
 <?php
