@@ -1,6 +1,6 @@
 <?php
 # dvdisaster: English homepage translation
-# Copyright (C) 2004-2008 Carsten Gnörlich
+# Copyright (C) 2004-2009 Carsten Gnörlich
 #
 # UTF-8 trigger: äöüß 
 #
@@ -32,6 +32,7 @@ The current version of dvdisaster contains screen texts in the following languag
 <tr><td></td><td>English</td><td>--</td><td>complete</td></tr>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>German</td><td>--</td><td>complete</td></tr>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Italian</td><td>--</td><td>upto version 0.65</td></tr>
+<tr><td></td><td>Portuguese</td><td>--</td><td>complete</td></tr>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Russian</td><td>--</td><td>complete</td></tr>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Swedish</td><td>--</td><td>upto version 0.70</td></tr>
 </table><p>
@@ -56,7 +57,7 @@ try the following:<p>
 
 <b><a name="media">2.2 Which media types are supported?</a></b><p>
 
-dvdisaster supports (re-)writeable CD and DVD media. <br>
+dvdisaster supports (re-)writeable CD, DVD and BD media. <br>
 Media containing multiple sessions or copy protections can <i>not</i> be used.<p>
 
 Usable media by type:<p>
@@ -94,8 +95,16 @@ Remedy: Determine the image size from the ISO/UDF file system or the ECC/RS02 da
 recognition as noted above.</li>
 </ul>
 
+<b>BD-R, BD-RW</b><p>
+
+<ul>
+<li>No limitations are known so far, but very few results
+on the two-layered type (the 50GB variety) have been reported yet.
+</li>
+</ul>
+
 <b>Not usable types</b> (image can not be extracted):<p>
-DVD-ROM (pressed DVDs), CD-Audio and CD-Video.
+BD-ROM (pressed BDs), DVD-ROM (pressed DVDs), CD-Audio and CD-Video.
 
 <div align=right><a href="#top">&uarr;</a></div><p>
 
