@@ -1,6 +1,6 @@
 <?php
 # dvdisaster: Russian homepage translation
-# Copyright (C) 2007-2008 Igor Gorbounov
+# Copyright (C) 2007-2009 Igor Gorbounov
 #
 # UTF-8 trigger: äöüß 
 #
@@ -17,14 +17,14 @@ begin_page();
 
 <h3>Примеры исправления ошибок</h3>
 
-<?php begin_screen_shot("Чтение поврежденного носителя.","recover_linear.png"); ?>
+<?php begin_screen_shot("Чтение поврежденного носителя.","recover-linear.png"); ?>
    <b>Восстановление старого носителя.</b> Обработанный здесь носитель стал
    обесцвеченным и частично нечитаемым во внешней области. Проверка поверхности
    дает примерно 23.000 нечитаемых секторов из общего числа 342.000 секторов; отсюда
    поврежденных секторов около 7,2%.
 <?php end_screen_shot(); ?>
 
-<?php begin_screen_shot("Чтение поврежденного образа.","fix_image.png"); ?>
+<?php begin_screen_shot("Чтение поврежденного образа.","fix-image.png"); ?>
    <b>Восстановление поврежденного образа.</b> Полученный образ все еще неполный,
    поскольку примерно 23.000 секторов не могут быть прочитаны. Эти сектора теперь 
    реконструируются с помощью 
@@ -44,8 +44,8 @@ begin_page();
 Как и в случае обычного резервного копирования, ecc-данные необходимо создать
 <i>прежде</i>, чем носитель станет поврежденным.<p>
 
-Поэтому, если у Вас есть поврежденный носитель, но Вы для него не создавали ecc-данных - увы, 
-Ваши данные, вероятно, потеряны.<p>
+Поэтому, если у вас есть поврежденный носитель, но вы для него не создавали ecc-данных - увы, 
+ваши данные, вероятно, потеряны.<p>
 
 
 <a href="index20.php">Почему проверки качества носителя недостаточно...</a>
