@@ -1,6 +1,6 @@
 <?php
 # dvdisaster: Russian homepage translation
-# Copyright (C) 2007-2008 Igor Gorbounov
+# Copyright (C) 2007-2009 Igor Gorbounov
 #
 # UTF-8 trigger: äöüß 
 #
@@ -33,6 +33,7 @@ begin_page();
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Немецкий</td><td>--</td><td>полный</td></tr>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Итальянский</td><td>--</td><td>до версии 0.65</td></tr>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Русский</td><td>--</td><td>полный</td></tr>
+<tr><td>&nbsp;&nbsp;&nbsp;</td><td>Португальский</td><td>--</td><td>полный<td></tr>
 <tr><td>&nbsp;&nbsp;&nbsp;</td><td>Шведский</td><td>--</td><td>до версии 0.70</td></tr>
 </table><p>
 
@@ -95,7 +96,7 @@ dvdisaster поддерживает (пере-)записываемые CD и DV
 </ul>
 
 <b>Непригодные типы</b> (образ нельзя извлечь):<p>
-DVD-ROM (печатаемые DVDs), CD-Audio и CD-Video.
+DVD-ROM (печатаемые DVD), CD-Audio и CD-Video.
 
 <div align=right><a href="#top">&uarr;</a></div><p>
 
@@ -123,7 +124,7 @@ Windows NT и ее наследники обеспечивают схожую ф
 
 <table>
 <tr valign=top><td>+</td><td>Для доступа к приводам не требуются прав администратора.</td></tr>
-<tr valign=top><td>-</td><td>ASPI не очень хорошо отображет приводы на буквы устройств Windows.</td></tr>
+<tr valign=top><td>-</td><td>ASPI не очень хорошо отображает приводы на буквы устройств Windows.</td></tr>
 <tr valign=top><td>-</td><td>Некоторые программы для записи на CD/DVD устанавливают
 свои собственные драйвера ASPI. Установка еще одного драйвера ASPI может вызвать конфликт 
 с существующим ПО.</td></tr>
