@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -26,33 +26,6 @@
  #include <winnls.h>
 #endif
 
-/*
- * A special marker for a unread sector 
- */
-
-void PrepareDeadSector()
-{   char *end_marker;
-    int end_length; 
-
-    if(!Closure->deadSector)
-      Closure->deadSector = g_malloc(2048);
-
-    if(Closure->fillUnreadable >= 0)
-      memset(Closure->deadSector, Closure->fillUnreadable, 2048);
-    else
-    {  memset(Closure->deadSector, 0 ,2048);
-       g_sprintf(Closure->deadSector,
-		 "dvdisaster dead sector marker\n"
-		 "This sector could not be read from the image.\n"
-		 "Its contents have been substituted by the dvdisaster read routine.\n");
-
-       end_marker = "dvdisaster dead sector end marker\n";
-       end_length = strlen(end_marker);
-       memcpy(Closure->deadSector+2046-end_length, end_marker, end_length); 
-    }
-}
-
-
 /*
  * Create the error correction file
  */
@@ -124,6 +97,7 @@ typedef enum
    MODE_BYTESET, 
    MODE_COPY_SECTOR,
    MODE_CMP_IMAGES,
+   MODE_DEBUG_MAINT1,
    MODE_ERASE, 
    MODE_LIST_ASPI,
    MODE_MARKED_IMAGE,
@@ -147,11 +121,13 @@ typedef enum
    MODIFIER_DAO, 
    MODIFIER_DEBUG,
    MODIFIER_DEFECTIVE_DUMP,
+   MODIFIER_DRIVER,
    MODIFIER_EJECT,
    MODIFIER_FILL_UNREADABLE,
    MODIFIER_IGNORE_FATAL_SENSE,
    MODIFIER_INTERNAL_REREADS,
    MODIFIER_QUERY_SIZE,
+   MODIFIER_NEW_DS_MARKER,
    MODIFIER_RANDOM_SEED,
    MODIFIER_READ_ATTEMPTS,
    MODIFIER_READ_MEDIUM,
@@ -163,14 +139,14 @@ typedef enum
    MODIFIER_SPINUP_DELAY, 
    MODIFIER_SPLIT_FILES,
    MODIFIER_TRUNCATE,
-   MODIFIER_VERSION
+   MODIFIER_VERSION,
 } run_mode;
 
 int main(int argc, char *argv[])
 {  int mode = MODE_NONE; 
    int sequence = MODE_NONE;
+   int devices_queried = FALSE;
    char *debug_arg = NULL;
-   char *default_device;
    char *read_range = NULL;
 #ifdef WITH_NLS_YES
    char *locale_test;
@@ -179,7 +155,7 @@ int main(int argc, char *argv[])
    char src_locale_path[strlen(SRCDIR)+10];
   #endif
  #endif /* WITH_EMBEDDED_SRC_PATH_YES */
- #ifdef SYS_MINGW
+#if defined(SYS_MINGW) || defined(SYS_DARWIN)
    char *bin_locale_path = NULL;
  #endif
 #endif
@@ -188,10 +164,14 @@ int main(int argc, char *argv[])
     atexit(check_memleaks);
 #endif
 
+    /*** Do this early so that we can use mutexes */
+
+    g_thread_init(NULL);
+
 #ifdef SYS_MINGW
     /*** Create a named mutex so that the installer can detect
 	 that we are running. A malicious user may cause this 
-	 to fail be reserving the mutex for himself, 
+	 to fail by reserving the mutex for himself, 
 	 so we do not care if the mutex can not be created. */
 
     CreateMutex(NULL, FALSE, "dvdisaster");
@@ -203,7 +183,6 @@ int main(int argc, char *argv[])
 
     /*** Setup for multithreading */
 
-    g_thread_init(NULL);
     Closure->mainThread = g_thread_self();
 
     /*** Setup the locale.
@@ -221,6 +200,9 @@ int main(int argc, char *argv[])
 	  and set the respective environment variables. */
 
        lang_id = GetUserDefaultLangID(); 
+       PrintLog("GetUserDefaultLangID = %x; Primary: %d, Sub: %d\n",
+		lang_id, PRIMARYLANGID(lang_id), SUBLANGID(lang_id));
+
        switch(PRIMARYLANGID(lang_id))
        {  case LANG_CZECH:
 	     g_setenv("LANG", "cs_CZ", 1);
@@ -249,6 +231,17 @@ int main(int argc, char *argv[])
 #endif
 	     break;
 
+          case LANG_PORTUGUESE:
+	    if(SUBLANGID(lang_id) == SUBLANG_PORTUGUESE_BRAZILIAN)
+	    {  g_setenv("LANG", "pt_BR", 1);
+#ifdef WIN_CONSOLE
+	       g_setenv("OUTPUT_CHARSET", "CP860", 1);
+#else
+	       g_setenv("OUTPUT_CHARSET", "CP1251", 1);
+#endif
+	    }
+            break;
+
           case LANG_RUSSIAN:
 	     g_setenv("LANG", "ru_RU", 1);
 #ifdef WIN_CONSOLE
@@ -280,8 +273,6 @@ int main(int argc, char *argv[])
 #ifdef WITH_EMBEDDED_SRC_PATH_YES
     g_sprintf(src_locale_path,"%s/locale",SRCDIR);
     bindtextdomain("dvdisaster", src_locale_path);
-//printf("testing src %s\n", src_locale_path);
-
 #endif /* WITH_EMBEDDED_SRC_PATH_YES */
 
     /* TRANSLATORS: 
@@ -298,19 +289,22 @@ int main(int argc, char *argv[])
        if(getcwd(buf, 256))
        {  char locale_path[strlen(buf)+20];
           g_sprintf(locale_path,"%s/locale", buf);
-//printf("testing cwd %s\n", locale_path);
           bindtextdomain("dvdisaster", locale_path);
 	  locale_test = gettext("test phrase for verifying the locale installation");
        }
     }
 #endif
 
-#ifdef SYS_MINGW
+#if defined(SYS_MINGW) || defined(SYS_DARWIN)
     /* Try the directory where our executable comes from.
-       This only possible under Windows, and should cover all cases. */
+       This is only possible under Windows and Mac OS, 
+       and should cover all cases. */
 
+#ifdef SYS_MINGW
     bin_locale_path = g_strdup_printf("%s\\locale", Closure->binDir);
-//printf("testing bin %s\n", bin_locale_path);
+#else
+    bin_locale_path = g_strdup_printf("%s/locale", Closure->binDir);
+#endif
     bindtextdomain("dvdisaster", bin_locale_path);
     locale_test = gettext("test phrase for verifying the locale installation");
     g_free(bin_locale_path);
@@ -318,13 +312,14 @@ int main(int argc, char *argv[])
 
     /* Last resort: fall back to global locale */
 
-//    if(strcmp(locale_test, "ok"))
-//      printf("fallback used\n");
-
     if(strcmp(locale_test, "ok"))
       bindtextdomain("dvdisaster", LOCALEDIR);  
 #endif /* WITH_NLS_YES */
 
+   /*** Create some localized file name presets */
+
+   LocalizedFileDefaults();   
+
    /*** Collect the Method list */
 
    CollectMethods();
@@ -335,13 +330,6 @@ int main(int argc, char *argv[])
    if(sizeof(EccHeader) != 4096)
      Stop("sizeof(EccHeader) is %d, but must be 4096.\n", sizeof(EccHeader));
 
-   /*** Determine the default device (OS dependent!) */
-
-   default_device = DefaultDevice();
-   if(!Closure->device)
-        Closure->device = default_device;
-   else g_free(default_device);
-
    /*** Parse the options */
    
    for(;;)
@@ -359,8 +347,10 @@ int main(int argc, char *argv[])
 	{"create", 0, 0, 'c'},
 	{"dao", 0, 0, MODIFIER_DAO },
 	{"debug", 0, 0, MODIFIER_DEBUG },
+	{"debug1", 1, 0, MODE_DEBUG_MAINT1 },
 	{"defective-dump", 1, 0, MODIFIER_DEFECTIVE_DUMP },
 	{"device", 0, 0, 'd'},
+	{"driver", 1, 0, MODIFIER_DRIVER },
         {"ecc", 1, 0, 'e'},
 	{"eject", 0, 0, MODIFIER_EJECT },
 	{"erase", 1, 0, MODE_ERASE },
@@ -377,6 +367,7 @@ int main(int argc, char *argv[])
 	{"marked-image", 1, 0, MODE_MARKED_IMAGE },
 	{"merge-images", 1, 0, MODE_MERGE_IMAGES },
 	{"method", 2, 0, 'm' },
+	{"new-ds-marker", 0, 0, MODIFIER_NEW_DS_MARKER },
         {"prefix", 1, 0, 'p'},
 	{"query-size", 1, 0, MODIFIER_QUERY_SIZE },
 	{"random-errors", 1, 0, MODE_RANDOM_ERR },
@@ -437,7 +428,7 @@ int main(int argc, char *argv[])
 	           if(Closure->sectorSkip<0) Closure->sectorSkip = 0;
 		   break;
 	 case 'l': mode = MODE_LIST_ASPI; break;
-         case 'm': if(optarg) 
+         case 'm': if(optarg && strlen(optarg) == 4) 
 	           {  g_free(Closure->methodName);
 	              Closure->methodName = g_strdup(optarg); 
                    }
@@ -493,8 +484,8 @@ int main(int argc, char *argv[])
 	   Closure->cacheMB = atoi(optarg);
 	   if(Closure->cacheMB <   8) 
 	     Stop(_("--cache-size must at least be 8MB; 16MB or higher is recommended.")); 
-	   if(Closure->cacheMB > 2048) 
-	     Stop(_("--cache-size maximum is 2048MB.")); 
+	   if(Closure->cacheMB > 8192) 
+	     Stop(_("--cache-size maximum is 8192MB.")); 
 	   break;
          case MODIFIER_CDUMP:
 	   Closure->debugCDump = TRUE;
@@ -505,6 +496,16 @@ int main(int argc, char *argv[])
          case MODIFIER_EJECT: 
 	   Closure->eject = 1; 
 	   break;
+	 case MODIFIER_DRIVER: /* currently undocumented feature */
+#if defined(SYS_LINUX)
+	   if(optarg && !strcmp(optarg,"sg"))
+	      Closure->useSGioctl = TRUE;
+	   else
+	      Stop(_("Valid args for --driver: sg"));
+#else
+	   Stop(_("--driver is only supported on GNU/Linux"));
+#endif
+	   break;
          case MODIFIER_FILL_UNREADABLE:
 	   if(optarg) Closure->fillUnreadable = strtol(optarg, NULL, 0);
 	   break;
@@ -542,6 +543,9 @@ int main(int argc, char *argv[])
 	    }
 	 }
 	   break;
+	 case MODIFIER_NEW_DS_MARKER:
+	    Closure->dsmVersion = 1;
+	    break;
          case MODIFIER_QUERY_SIZE:
 	        if(!strcmp(optarg, "drive")) Closure->querySize = 0;
 	   else if(!strcmp(optarg, "udf"))   Closure->querySize = 1;
@@ -607,7 +611,8 @@ int main(int argc, char *argv[])
 	   Closure->driveSpeed = -atoi(optarg);
 	   break;
          case MODIFIER_VERSION:
-	   PrintCLI(_("\ndvdisaster version %s\n\n"), Closure->cookedVersion);
+	   PrintCLI(_("\ndvdisaster version %s build %d\n\n"), 
+		    Closure->cookedVersion, buildCount);
 	   FreeClosure();
 	   exit(EXIT_SUCCESS); 
 	   break;
@@ -623,6 +628,10 @@ int main(int argc, char *argv[])
 	   mode = MODE_COPY_SECTOR;
 	   debug_arg = g_strdup(optarg);
 	   break;
+         case MODE_DEBUG_MAINT1:
+	   mode = MODE_DEBUG_MAINT1;
+	   debug_arg = g_strdup(optarg);
+	   break;
          case MODE_ERASE: 
 	   mode = MODE_ERASE;
 	   debug_arg = g_strdup(optarg);
@@ -693,8 +702,6 @@ int main(int argc, char *argv[])
           break;
      }
 	  
-   PrepareDeadSector();
-
 #ifdef WIN_CONSOLE
    if(mode != MODE_SIGN && !VerifySignature())
    {  char version[80];
@@ -707,6 +714,10 @@ int main(int argc, char *argv[])
    }
 #endif
 
+   /*** CPU type detection. */
+
+   Closure->useSSE2 = ProbeSSE2();
+
    /*** Parse the sector ranges for --read and --scan */
 
    if(read_range)
@@ -733,6 +744,14 @@ int main(int argc, char *argv[])
       Closure->imageName = ApplyAutoSuffix(Closure->imageName, "iso");
    }
 
+   /*** Determine the default device (OS dependent!) if none
+        has been specified on the command line. */
+
+   if(!Closure->device)
+   {  Closure->device = DefaultDevice();
+      devices_queried = TRUE;
+   }
+
    /*** Dispatch action depending on mode.
         The major modes can be executed in sequence, 
 	but not all combinations may be really useful.
@@ -780,6 +799,10 @@ int main(int argc, char *argv[])
 	 CopySector(debug_arg);
 	 break;
 
+      case MODE_DEBUG_MAINT1:
+ 	 Maintenance1(debug_arg);
+	 break;
+
       case MODE_ERASE:
          Erase(debug_arg);
 	 break;
@@ -883,6 +906,7 @@ int main(int argc, char *argv[])
 	     "  --auto-suffix          - automatically add .iso and .ecc file suffixes\n"
 	     "  --cache-size n         - image cache size in MB during -c mode (default: 32MB)\n"
 	     "  --dao                  - assume DAO disc; do not trim image end\n"
+	     "  --defective-dump d     - directory for saving incomplete raw sectors\n"
 	     "  --eject                - eject medium after successful read\n"
 	     "  --fill-unreadable n    - fill unreadable sectors with byte n\n"
 	     "  --ignore-fatal-sense   - continue reading after potentially fatal error conditon\n"
@@ -903,7 +927,6 @@ int main(int argc, char *argv[])
              "  --cdump           - creates C #include file dumps instead of hexdumps\n" 
 	     "  --compare-images a,b  - compare sectors in images a and b\n"
 	     "  --copy-sector a,n,b,m - copy sector n from image a to sector m in image b\n"
-	     "  --defective-dump p  creates C #include file dumps for unrecoverable sectors\n"
 	     "  --erase sector    - erase the given sector\n"
 	     "  --erase n-m       - erase sectors n - m, inclusively\n"
 	     "  --marked-image n  - create image with n marked random sectors\n"
@@ -937,7 +960,6 @@ int main(int argc, char *argv[])
 
    if(mode == MODE_NONE)
    {  
-
       if(Closure->screenShotMode)
       {  GPtrArray *a=Closure->deviceNames;
 	 int i;
@@ -948,6 +970,16 @@ int main(int argc, char *argv[])
 	 }
       }
 
+
+      /* We need to query devices in order to build
+	 the drop-down menu.*/
+
+      if(!devices_queried)
+      {  if(Closure->device)
+	    g_free(Closure->device);
+	 Closure->device = DefaultDevice();
+      }
+
       Closure->guiMode = TRUE;
       ReadDotfile();
       CreateMainWindow(&argc, &argv);
