@@ -1,6 +1,5 @@
-
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -99,7 +98,7 @@
 #define DVD_SL_SIZE      2295104  /* DVD+R/RW size used at least common denominator */
 #define DVD_DL_SIZE 	 4171712  /* also seen: 4148992 4173824  */
 #define BD_SL_SIZE      11826176
-#define BD_DL_SIZE	21826176  /* whats the right value here? */
+#define BD_DL_SIZE	23652352
 
 /* Maximum accepted media sizes (in 2K sectors) */
 
@@ -154,6 +153,7 @@ typedef struct _GlobalClosure
    int spinupDelay;     /* Seconds to wait for drive to spin up */
    int truncate;        /* confirms truncation of large images */
    int noTruncate;      /* do not truncate image at the end */
+   int dsmVersion;      /* 1 means new style dead sector marker */
    int unlinkImage;     /* delete image after ecc file creation */
    int driveSpeed;      /* currently unused */
    int debugMode;       /* may activate additional features */
@@ -178,8 +178,10 @@ typedef struct _GlobalClosure
    int pauseDuration;   /* duration of pause in minutes */
    int pauseEject;      /* Eject medium during pause */
    int ignoreFatalSense;/* Continue reading after potential fatal sense errors */
+   int useSSE2;         /* TRUE means to use SSE2 version of the codec. */
+   int useSGioctl;      /* Use the generic SCSI ioctl instead of CDROM one on Liux */
   
-   char *deadSector;    /* Copy of our "dead sector" marker. */
+   char *homeDir;       /* path to users home dir */
    char *dotFile;       /* path to .dvdisaster file */
    char *logFile;       /* path to logfile */
    int  logFileEnabled; /* logfile enabled */
@@ -212,13 +214,15 @@ typedef struct _GlobalClosure
 
    /*** GUI-related things */
 
-   int guiMode;         /* TRUE if GUI is active */
-   int stopActions;     /* crude method to stop ongoing action(s) */
+   int guiMode;              /* TRUE if GUI is active */
+   int stopActions;          /* crude method to stop ongoing action(s) */
+   int noMissingWarnings;    /* suppress warnings about inconsistent missing sectors */
 
    GtkWidget *logWidget;     /* Dialog for the log display */
    GtkScrolledWindow *logScroll; /* and its scrolled window */
    GtkTextBuffer *logBuffer; /* Text buffer for the log output */
    GString *logString;       /* holds logging output for current action */
+   GMutex *logLock;          /* protects the logString */
 
    /*** Widgets of the main window */
 
@@ -290,9 +294,8 @@ typedef struct _GlobalClosure
    GtkWidget *readLinearErrors;
    GtkWidget *readLinearFootline;
    GtkWidget *readLinearFootlineBox;
-   int    checkCrc;               /* these are passed between threads and */
-   gint64 crcErrors, readErrors;  /* must therefore be global */
-
+   gint64 crcErrors, readErrors;  /* these are passed between threads and must therefore be global */
+   int    eccType;                /* type of ecc data provided while reading/scanning */
    /*** Widgets for the adaptive reading action */
 
    GtkWidget *readAdaptiveHeadline;
@@ -338,6 +341,7 @@ typedef struct _ImageInfo
    gint64 crcErrors;                 /* sectors with CRC32 errors */
    int inLast;                       /* bytes in last sector */
    unsigned char mediumFP[16];       /* md5sum of first sector */
+   int fpValid;                      /* TRUE if above is a valid md5sum */
    unsigned char mediumSum[16];      /* complete md5sum of whole medium */
 } ImageInfo;
 
@@ -386,7 +390,7 @@ typedef struct _EccHeader
    gint32 inLast;              /* bytes contained in last sector */
    gint8 padding[3976];        /* pad to 4096 bytes: room for future expansion */
 
-  /* Note: Bytes 2048 and up are currently used by the RS02 codec
+  /* Note: Bytes 2048 and up are currently used by the RS02/RS03 codec
            for a copy of the first ecc blocks CRC sums. */
 } EccHeader;
 
@@ -395,13 +399,12 @@ typedef struct _EccHeader
  ***/
 
 struct _RawBuffer *rawbuffer_forward;
+struct _DefectiveSectorHeader *dsh_forward;
 
 /***
  *** dvdisaster.c
  ***/
 
-void PrepareDeadSector(void);
-
 void CreateEcc(void);
 void FixEcc(void);
 void Verify(void);
@@ -423,11 +426,18 @@ Bitmap* CreateBitmap0(int);
 int CountBits(Bitmap*);
 void FreeBitmap(Bitmap*);
 
+/***
+ *** build.h
+ ***/
+
+int buildCount;
+
 /***
  *** closure.c
  ***/
 
 void InitClosure(void);
+void LocalizedFileDefaults(void);
 void UpdateMarkup(char**, GdkColor*);
 void DefaultColors(void);
 void ClearCrcCache(void);
@@ -528,6 +538,21 @@ Bitmap* SimulateDefects(gint64);
 void TruncateImage(char*);
 void ZeroUnreadable(void);
 
+/***
+ *** ds-marker.c
+ ***/
+
+enum
+{  SECTOR_PRESENT,
+   SECTOR_MISSING,
+   SECTOR_MISSING_DISPLACED,
+   SECTOR_MISSING_WRONG_FP
+};
+
+void CreateMissingSector(unsigned char*, gint64, unsigned char*, gint64, char*);
+int CheckForMissingSector(unsigned char*, gint64, unsigned char*, gint64);
+void ExplainMissingSector(unsigned char*, gint64, int, int);
+
 /***
  *** endian.c
  ***/
@@ -535,6 +560,7 @@ void ZeroUnreadable(void);
 guint32 SwapBytes32(guint32);
 guint64 SwapBytes64(guint64);
 void    SwapEccHeaderBytes(EccHeader*);
+void    SwapDefectiveHeaderBytes(struct _DefectiveSectorHeader*);
 
 /***
  *** file.c
@@ -607,6 +633,9 @@ typedef struct _ReedSolomonTables
    gint32 primElem;      /* primitive field element */
    gint32 nroots;        /* degree of RS generator polynomial */
    gint32 ndata;         /* data bytes per ecc block */
+   gint32 shiftInit;     /* starting value for iteratively processing parity */
+
+   guint8 *bLut[GF_FIELDSIZE];  /* experimental 8bit lookup table */
 } ReedSolomonTables;
 
 GaloisTables* CreateGaloisTables(gint32);
@@ -682,8 +711,8 @@ void CreateIconFactory();
 LargeFile *LargeOpen(char*, int, mode_t);
 int LargeSeek(LargeFile*, gint64);
 int LargeEOF(LargeFile*);
-int LargeRead(LargeFile*, void*, size_t);
-int LargeWrite(LargeFile*, void*, size_t);
+ssize_t LargeRead(LargeFile*, void*, size_t);
+ssize_t LargeWrite(LargeFile*, void*, size_t);
 int LargeClose(LargeFile*);
 int LargeTruncate(LargeFile*, gint64);
 int LargeStat(char*, gint64*);
@@ -737,6 +766,17 @@ void DefaultLogFile();
 void VPrintLogFile(char*, va_list);
 void PrintLogFile(char*, ...);
 
+/***
+ *** maintenance.c
+ ***
+ *
+ * Provides a context for running testing functions
+ * within the usual program context.
+ * Only for debugging / development purposes.
+ */
+
+void Maintenance1(char*);
+
 /***
  *** main-window.c
  ***/
@@ -946,7 +986,7 @@ void FreeRawEditorContext(void*);
  *** raw-sector-cache.c
  ***/
 
-typedef struct _dsh
+typedef struct _DefectiveSectorHeader
 {  unsigned char mediumFP[16];       /* Medium fingerprint */
    gint64 lba;                       /* LBA of this sector */
    gint32 sectorSize;                /* Sector size in bytes */
@@ -968,6 +1008,10 @@ void ReadDefectiveSectorFile(DefectiveSectorHeader *, struct _RawBuffer*, char*)
  *** read-linear.c
  ***/
 
+enum
+{ ECC_NONE, ECC_RS01, ECC_RS02
+};
+
 void ReadMediumLinear(gpointer);
 
 /***
@@ -1117,23 +1161,11 @@ gint64 CurrentImageCapacity(void);
 int SendReadCDB(char*, unsigned char*, unsigned char*, int, int);
 
 /***
- *** rs-encoder.c
+ *** rs-encoder.c and friends
  ***/
 
-typedef struct _ReedSolomonEncoder
-{  GaloisTables *gfTables;
-   ReedSolomonTables *rsTables;
-
-   guint64 chunkSize;
-   int shiftPtr;
-} ReedSolomonEncoder;
-
-ReedSolomonEncoder *CreateRSEncoder(ReedSolomonTables*, guint64);
-void FreeReedSolomonEncoder(ReedSolomonEncoder*);
-
-void ResetReedSolomonEncoder(ReedSolomonEncoder*, unsigned char*);
-void EncodeNextLayer(ReedSolomonEncoder*, unsigned char*, unsigned char*, guint64);
-
+void EncodeNextLayer(ReedSolomonTables*, unsigned char*, unsigned char*, guint64, int);
+int ProbeSSE2(void);
 
 /***
  *** show-manual.c
