@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -120,3 +120,12 @@ void SwapEccHeaderBytes(EccHeader *eh)
   print_ecc_header(eh);
 #endif
 }
+
+void SwapDefectiveHeaderBytes(DefectiveSectorHeader *dsh)
+{  
+  dsh->lba        = SwapBytes64(dsh->lba);
+  dsh->sectorSize = SwapBytes32(dsh->sectorSize);
+  dsh->properties = SwapBytes32(dsh->properties);
+  dsh->dshFormat  = SwapBytes32(dsh->dshFormat);
+  dsh->nSectors   = SwapBytes32(dsh->nSectors);
+}
