@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -80,6 +80,8 @@ static gchar* os_path(char *path_in)
    {  errno = EINVAL;
       return NULL;
    }
+   
+   REMEMBER(cp_path);
    return cp_path;
 }
 
@@ -345,8 +347,8 @@ int LargeEOF(LargeFile *lf)
  * Reading in segmented files
  */
 
-int LargeRead(LargeFile *lf, void *buf, size_t count)
-{  int n;
+ssize_t LargeRead(LargeFile *lf, void *buf, size_t count)
+{  ssize_t n;
 
    /* Simple unsegmented case */  
 
@@ -459,14 +461,26 @@ static void insert_buttons(GtkDialog *dialog)
 			 GTK_STOCK_CANCEL, 0, NULL);
 } 
 
-static int xwrite(int fdes, void *buf_base, size_t count)
+static ssize_t xwrite(int fdes, void *buf_base, size_t count)
 {  unsigned char *buf = (unsigned char*)buf_base;
-   int total = 0;
+   ssize_t total = 0;
 
    /* Simply fail when going out of space in command line mode */
 
    if(!Closure->guiMode)
-     return write(fdes, buf, count);
+   {  while(count)
+      {  ssize_t n = write(fdes, buf, count);
+      
+	 if(n<=0) return total;  /* error occurred */
+
+	 if(n>0)  /* write at least partially successful */
+	 {  total += n;
+	    count -= n;
+	    buf   += n;
+	 }
+      }
+      return total;
+   }
 
    /* Give the user a chance to free more space in GUI mode.
       When running out of space, the last write() may complete
@@ -474,9 +488,9 @@ static int xwrite(int fdes, void *buf_base, size_t count)
       until a real error hits (n = -1). */
 
    while(count)
-   {  int n = write(fdes, buf, count);
+   {  ssize_t n = write(fdes, buf, count);
 
-      if(n < 0) /* error occurred */
+      if(n <= 0) /* error occurred */
       {  int answer; 
 
 	 if(errno != ENOSPC) return total;
@@ -499,8 +513,8 @@ static int xwrite(int fdes, void *buf_base, size_t count)
    return total;
 }
 
-int LargeWrite(LargeFile *lf, void *buf, size_t count)
-{  int n;
+ssize_t LargeWrite(LargeFile *lf, void *buf, size_t count)
+{  ssize_t n;
 
    /* Simple unsegmented case */  
 
