@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -65,8 +65,10 @@ static void action_cb(GtkWidget *widget, gpointer data)
       /* Clear the log buffer, request new log file time stamp */
       
       if(action != ACTION_CREATE_CONT)
-      {  g_string_truncate(Closure->logString, 0);
+      {  g_mutex_lock(Closure->logLock);
+	 g_string_truncate(Closure->logString, 0);
          g_string_printf(Closure->logString, _("dvdisaster-%s log\n"),VERSION);
+	 g_mutex_unlock(Closure->logLock);
 	 Closure->logFileStamped = FALSE;
       }
 
@@ -95,6 +97,14 @@ static void action_cb(GtkWidget *widget, gpointer data)
       {  remove_the_00s(Closure->imageName);
 	 remove_the_00s(Closure->eccName);
       }
+
+      /* Reset warnings which may be temporarily disabled during an action */
+
+      Closure->noMissingWarnings = FALSE;
+
+      /* Do not queue up stop actions */
+
+      Closure->stopActions = FALSE;
    }
 
    /* Dispatch action */
@@ -378,7 +388,6 @@ void CreateMainWindow(int *argc, char ***argv)
 
     g_signal_connect(window, "destroy", G_CALLBACK(destroy_cb), NULL);
 
-
     /*** Validate our binary directory */
 
 #ifdef SYS_MINGW
