@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -167,8 +167,11 @@ static void log_window_vprintf(char *format, va_list argp)
 
    tmp = g_strdup_vprintf(format, argp);
    utf_tmp = g_locale_to_utf8(tmp, -1, NULL, NULL, NULL);
+
+   g_mutex_lock(Closure->logLock);
    g_string_append(Closure->logString, utf_tmp);
    clamp_gstring(Closure->logString);
+   g_mutex_unlock(Closure->logLock);
 
    UpdateLog();
    
@@ -179,8 +182,10 @@ static void log_window_vprintf(char *format, va_list argp)
 static void log_window_append(char *text)
 {  char *utf_tmp = g_locale_to_utf8(text, -1, NULL, NULL, NULL);
 
+   g_mutex_lock(Closure->logLock);
    g_string_append(Closure->logString, utf_tmp);
    clamp_gstring(Closure->logString);
+   g_mutex_unlock(Closure->logLock);
 
    UpdateLog();
    
@@ -197,7 +202,7 @@ static void print_greetings(FILE *where)
    if(greetings_shown) return;
 
    greetings_shown = 1;
-   g_fprintf(where, _("dvdisaster-%s%sCopyright 2004-2008 Carsten Gnoerlich.\n"),
+   g_fprintf(where, _("dvdisaster-%s%sCopyright 2004-2009 Carsten Gnoerlich.\n"),
 	     VERSION, strstr(VERSION,"pl") ? " " : "  ");
    /* TRANSLATORS: Excluding all kinds of warranty might be harmful under your
       legislature. If in doubt, just translate the following like "This is free
@@ -247,7 +252,8 @@ void PrintCLI(char *format, ...)
  */
 
 void PrintProgress(char *format, ...)
-{  char msg[256];
+{  static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+   char msg[256];
    va_list argp;
    int n;
 
@@ -273,9 +279,11 @@ void PrintProgress(char *format, ...)
    if(strchr(msg, '\n'))
       g_fprintf(stderr, "%s", msg);
    else
-   {  Closure->bs[n] = 0;
+   {  g_static_mutex_lock(&mutex);
+      Closure->bs[n] = 0;
       g_fprintf(stderr, "%s%s", msg, Closure->bs);
       Closure->bs[n] = '\b';
+      g_static_mutex_unlock(&mutex);
    }
 
    fflush(stderr);   /* at least needed for Windows */
@@ -755,7 +763,6 @@ void SetLabelText(GtkLabel *label, char *format, ...)
    }
    else li->text = g_locale_to_utf8("(null)", -1, NULL, NULL, NULL);
    va_end(argp);
-
    g_idle_add(label_idle_func, li);
 }
 
