@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -355,7 +355,7 @@ static int check_for_sync_pattern(unsigned char *new_frame)
 int CheckEDC(unsigned char *cd_frame, int xa_mode)
 { unsigned int expected_crc, real_crc;
 
-   /* Get CRC from CD frame (byte position 2064) */
+   /* XA: Get CRC from byte position 2072 */
 
    if(xa_mode)
    {  expected_crc =  cd_frame[2072] << 24
@@ -363,7 +363,7 @@ int CheckEDC(unsigned char *cd_frame, int xa_mode)
                     | cd_frame[2074] <<  8
                     | cd_frame[2075];
    }
-   else
+   else /* Get CRC from byte position 2064 */
    {  expected_crc =  cd_frame[0x810] << 24
                     | cd_frame[0x811] << 16
                     | cd_frame[0x812] <<  8
@@ -493,7 +493,7 @@ int ValidateRawSector(RawBuffer *rb, unsigned char *frame, char *msg)
 
    /* See if the buffer was returned unchanged. */
 
-   if(!memcmp(frame, Closure->deadSector, 2048))
+   if(CheckForMissingSector(frame, rb->lba, NULL, 0) != SECTOR_PRESENT)
    {  RememberSense(3, 255, 0);  /* No data returned */
       return FALSE;
    }
@@ -829,7 +829,7 @@ int TryCDFrameRecovery(RawBuffer *rb, unsigned char *outbuf)
 
    /* See if the buffer was returned unchanged. */
 
-   if(!memcmp(new_frame, Closure->deadSector, 2048))
+   if(CheckForMissingSector(new_frame, rb->lba, NULL, 0) != SECTOR_PRESENT)
    {  RememberSense(3, 255, 0);  /* No data returned */
       return -1;
    }
@@ -871,16 +871,50 @@ int TryCDFrameRecovery(RawBuffer *rb, unsigned char *outbuf)
 
    UpdateFrameStats(rb);
 
-   /*** Cheap shot: See if we can recover the sector itself
+   /*** Cheap shots: See if we can recover the sector itself
 	(without using the more complex heuristics and other
 	sectors).
-        Note that we ignore the return value of iterative_recovery().
+        Note that we ignore the return value of IterativeLEC().
         If e.g. some parity bytes remain uncorrected we don't care
         as long as the EDC tells us that the user data part is okay. */
 
    memcpy(rb->recovered, new_frame, rb->sampleSize);
    memset(rb->byteState, 0, rb->sampleSize);
 
+   /* If the data section is unaffected by the error,
+      do not investigate further. */
+
+   if(CheckEDC(rb->recovered, rb->xaMode)
+      && CheckMSF(rb->recovered, rb->lba, STRICT_MSF_CHECK))
+   {
+       PrintCLIorLabel(Closure->status, 
+		       "Sector %lld: Good. Data section passes EDC test.\n",
+		       rb->lba);
+       memcpy(outbuf, rb->recovered+rb->dataOffset, 2048);
+       return 0;
+   }
+
+   /* Sometimes we have only errors in the sync pattern and the P/Q vectors,
+      but the data section will pass the EDC test. Try it. */
+
+   if(memcmp(rb->recovered, sync_pattern, 12))
+   {  memcpy(rb->recovered, sync_pattern, 12);
+
+      if(CheckEDC(rb->recovered, rb->xaMode)
+	 && CheckMSF(rb->recovered, rb->lba, STRICT_MSF_CHECK))
+      {
+	 PrintCLIorLabel(Closure->status, 
+			 "Sector %lld: Recovered in raw reader after correcting sync pattern.\n",
+			 rb->lba);
+	 
+	 memcpy(outbuf, rb->recovered+rb->dataOffset, 2048);
+	 return 0;
+      }
+   }
+
+
+   /* Try the simple iterative L-EC */
+
    IterativeLEC(rb);
 
    if(CheckEDC(rb->recovered, rb->xaMode)
@@ -904,6 +938,19 @@ int TryCDFrameRecovery(RawBuffer *rb, unsigned char *outbuf)
 
    /* The actual heuristics */
 
+#if 0
+   SmartLEC(rb);
+
+   if(CheckEDC(rb->recovered, rb->xaMode)
+      && CheckMSF(rb->recovered, rb->lba, STRICT_MSF_CHECK))
+   {  PrintCLIorLabel(Closure->status, 
+		      "Sector %lld: Recovered in raw reader by smart L-EC.\n",
+		      rb->lba);
+      memcpy(outbuf, rb->recovered+rb->dataOffset, 2048);
+      return 0; 
+   }
+#endif
+
    SearchPlausibleSector(rb, 0);
 
    if(CheckEDC(rb->recovered, rb->xaMode)
