@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -163,8 +163,15 @@ static void check_image(ecc_closure *ec)
    guint32 *crcptr;
    int last_percent, percent;
 
-   if(Closure->crcCache) /* Discard old CRC cache no matter what it contains */
-     ClearCrcCache();    /* We will create a new one a few lines below */
+   /* Discard old CRC cache no matter what it contains.
+    * We will create a new one a few lines below.
+    * Note that it is very unusual to augment an image with ecc data
+    * which was just from an actual medium, so optimizing for the cached
+    * CRCs is not necessary. 
+    */
+
+   if(Closure->crcCache) 
+     ClearCrcCache();    
 
    last_percent = 0;
    MD5Init(&image_md5);
@@ -176,7 +183,7 @@ static void check_image(ecc_closure *ec)
 
    for(sectors = 0; sectors < lay->dataSectors; sectors++)
    {  unsigned char buf[2048];
-      int expected,n;
+      int expected,n,err;
 
       if(Closure->stopActions) /* User hit the Stop button */
 	abort_encoding(ec, FALSE);
@@ -191,10 +198,26 @@ static void check_image(ecc_closure *ec)
       if(n != expected)
 	Stop(_("Failed reading sector %lld in image: %s"),sectors,strerror(errno));
 
-      if(!memcmp(buf, Closure->deadSector, n))
-	Stop(_("Image contains unread(able) sectors.\n"
-	       "Error correction information can only be\n"
-	       "appended to complete (undamaged) images.\n"));
+      /* Look for the dead sector marker */
+
+      err = CheckForMissingSector(buf, sectors, ii->fpValid ? ii->mediumFP : NULL, FINGERPRINT_SECTOR);
+      if(err != SECTOR_PRESENT)
+      {    if(err == SECTOR_MISSING)
+	    Stop(_("Image contains unread(able) sectors.\n"
+		   "Error correction information can only be\n"
+		   "appended to complete (undamaged) images.\n"));
+	 else
+	    Stop(_("Sector %lld in the image is marked unreadable\n"
+		   "and seems to come from a different medium.\n\n"
+		   "The image was probably mastered from defective content.\n"
+		   "For example it might contain one or more files which came\n"
+		   "from a damaged medium which was NOT fully recovered.\n" 
+		   "This means that some files may have been silently corrupted.\n\n"
+		   "Error correction information can only be\n"
+		   "appended to complete (undamaged) images.\n"));
+      }
+      
+      /* Update and cache the CRC sums */
 
       *crcptr++ = Crc32(buf, 2048);
       MD5Update(&image_md5, buf, n);
@@ -251,12 +274,16 @@ static void expand_image(ecc_closure *ec)
 
    last_percent = 0;
    for(sectors = 0; sectors < lay->eccSectors; sectors++)
-   {  int n;
+   {  unsigned char buf[2048];
+      int n;
 
       if(Closure->stopActions) /* User hit the Stop button */
 	abort_encoding(ec, TRUE);
 
-      n = LargeWrite(ii->file, Closure->deadSector, 2048);
+      CreateMissingSector(buf, lay->dataSectors+sectors, 
+			  ii->mediumFP, FINGERPRINT_SECTOR, 
+			  "RS02 generation placeholder");
+      n = LargeWrite(ii->file, buf, 2048);
       if(n != 2048)
 	Stop(_("Failed expanding the image: %s\n"), strerror(errno));
 
@@ -412,11 +439,13 @@ static void create_reed_solomon(ecc_closure *ec)
    int nroots = lay->nroots;
    int ndata  = lay->ndata;
    gint64 b_idx, block_idx[256]; 
-   int n_parity_blocks,n_layer_sectors;
-   int n_parity_bytes,n_layer_bytes;
+   guint64 n_parity_blocks,n_layer_sectors;
+   guint64 n_parity_bytes,n_layer_bytes;
+   guint64 si,chunk;
    int last_percent, percent, max_percent, progress;
-   int layer,chunk,i,j,k;
+   int layer,i,j,k;
    unsigned char *par_ptr;
+   int out_of_memory = 0;
 static gint32 *gf_index_of;    /* These need to be static globals */
 static gint32 *rs_gpoly;       /* for optimization reasons. */
 static gint32 *enc_alpha_to;
@@ -446,10 +475,10 @@ static gint32 *enc_alpha_to;
         The algorithm builds the parity file consecutively in chunks of n_parity_blocks.
         We use all the amount of memory allowed by cacheMB for caching the parity blocks. */
 
-   n_parity_blocks = (Closure->cacheMB<<20) / nroots;  /* 1 MB = 2^20 */
+   n_parity_blocks = ((guint64)Closure->cacheMB<<20) / (guint64)nroots;  /* 1 MB = 2^20 */
    n_parity_blocks >>= 1;                              /* two buffer sets for scrambling */
    n_parity_blocks &= ~0x7ff;                          /* round down to multiple of 2048 */
-   n_parity_bytes  = nroots * n_parity_blocks;
+   n_parity_bytes  = (guint64)nroots * n_parity_blocks;
 
    /* Each chunk of parity blocks is built iteratively by processing the data in layers
       (first all bytes at pos 0, then pos 1, until ndata layers have been processed).
@@ -463,8 +492,24 @@ static gint32 *enc_alpha_to;
    if(n_layer_sectors*2048 != n_parity_blocks)
      Stop("Internal error: parity blocks are not a multiple of sector size.\n");
 
-   ec->parity = g_malloc(n_parity_bytes);
-   ec->data   = g_malloc(n_layer_bytes);
+   ec->parity = g_try_malloc(n_parity_bytes);
+   ec->data   = g_try_malloc(n_layer_bytes);
+
+   /*** Create buffers for dividing the ecc information into nroots slices */
+
+   for(i=0; i<nroots; i++)
+   {  ec->slice[i] = g_try_malloc(n_layer_bytes);
+      if(!ec->slice[i])
+	 out_of_memory = 1;
+   }
+
+   if(out_of_memory || !ec->parity || !ec->data)
+   {  LargeTruncate(ec->ii->file, (gint64)(2048*ec->lay->dataSectors));
+      Stop(_("Failed allocating memory for I/O cache.\n"
+	     "Cache size is currently %d MB.\n"
+	     "Try reducing it.\n"),
+	   Closure->cacheMB);
+   }
 
    /*** Setup the block counters for mapping medium sectors to ecc blocks 
         The image is divided into ndata layers;
@@ -473,11 +518,6 @@ static gint32 *enc_alpha_to;
    for(b_idx=0, i=0; i<ndata; b_idx+=lay->sectorsPerLayer, i++)
      block_idx[i] = b_idx;
 
-   /*** Create buffers for dividing the ecc information into nroots slices */
-
-   for(i=0; i<nroots; i++)
-     ec->slice[i] = g_malloc(n_layer_bytes);
-
    /*** Initialize md5 contexts for checksumming the nroots slices */
 
    for(i=0; i<nroots; i++)
@@ -496,7 +536,7 @@ static gint32 *enc_alpha_to;
       the whole image has been processed. */
 
    for(chunk=0; chunk<lay->sectorsPerLayer; chunk+=n_layer_sectors) 
-   {  int actual_layer_bytes,actual_layer_sectors;
+   {  guint64 actual_layer_bytes,actual_layer_sectors;
       int sp;
 
       /* Prepare the parity data for the next chunk. */
@@ -527,7 +567,7 @@ static gint32 *enc_alpha_to;
 
          /* Read the next data sectors of this layer. */
 
-   	 for(i=0; i<actual_layer_sectors; i++)
+   	 for(si=0; si<actual_layer_sectors; si++)
 	 {  RS02ReadSector(ec->ii, lay, ec->data+offset, block_idx[layer]);
 	    block_idx[layer]++;
 	    offset += 2048;
@@ -535,10 +575,10 @@ static gint32 *enc_alpha_to;
 
 	 /* Now process the data bytes of the current layer. */
 
-	 for(i=0; i<actual_layer_bytes; i++)
+	 for(si=0; si<actual_layer_bytes; si++)
 	 {  register int feedback;
 
-	    feedback = gf_index_of[ec->data[i] ^ par_idx[sp]];
+	    feedback = gf_index_of[ec->data[si] ^ par_idx[sp]];
 
 	    if(feedback != GF_ALPHA0) /* non-zero feedback term */
 	    {  register int spk = sp+1;
@@ -923,8 +963,8 @@ static gint32 *enc_alpha_to;
 
       par_ptr = ec->parity;
 
-      for(i=0; i<actual_layer_sectors; i++)
-      {  int idx = 2048*i;
+      for(si=0; si<actual_layer_sectors; si++)
+      {  guint64 idx = 2048*si;
 
 	 for(j=0; j<2048; j++, idx++)
 	 {  for(k=0; k<nroots; k++)
@@ -935,8 +975,8 @@ static gint32 *enc_alpha_to;
       for(k=0; k<nroots; k++)
       {  int idx=0;
 
-	for(i=0; i<actual_layer_sectors; i++, idx+=2048)
-	 {  gint64 s = RS02EccSectorIndex(lay, k, chunk + i);
+	for(si=0; si<actual_layer_sectors; si++, idx+=2048)
+	 {  gint64 s = RS02EccSectorIndex(lay, k, chunk + si);
 
 	    if(!LargeSeek(file, 2048*s))
 	      Stop(_("Failed seeking to sector %lld in image: %s"), s, strerror(errno));
