@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -133,7 +133,109 @@ int InquireDevice(DeviceHandle *dh, int probe_only)
 }
 
 /*
- * Ask drive for active profile; this give us a hint
+ * Some drives do not know about profiles.
+ * In theory only plain CD-ROM type drives have no choices between
+ * different profiles, but we must consider buggy firmware also.
+ * Do a quick and dirty differentiation between the possible formats.
+ */
+
+static int try_fallback_type_check(DeviceHandle *dh)
+{  AlignedBuffer *ab;
+   Sense *sense = &dh->sense;
+   unsigned char cmd[MAX_CDB_SIZE];
+   int length;
+
+   Verbose("# *** try_fallback_type_check(%s) ***\n", dh->devinfo);
+
+   ab = CreateAlignedBuffer(2048); 
+  
+   /*** If the medium is a BD, the following will succeed. */
+
+   /* Query length of returned data */
+
+   memset(cmd, 0, MAX_CDB_SIZE);
+   cmd[0] = 0xad;     /* READ DVD STRUCTURE */
+   cmd[1] = 0x01;     /* subcommand for BD */
+   cmd[6] = 0;        /* First layer */
+   cmd[7] = 0;        /* We want DI (disc information) */
+   cmd[8] = 0;        /* Allocation length */
+   cmd[9] = 2;
+
+   /*** Only a BD should respond positively here */
+   
+   Verbose("# BD: trying READ DVD with BD subcommand for size\n");
+   if(SendPacket(dh, cmd, 12, ab->buf, 2, sense, DATA_READ)<0)
+   {  
+      Verbose("# failed -> not a BD type medium.\n");
+      goto try_dvd;
+   }
+
+   /*** Some DVD drives ignore the media type 0x01 and return the dvd structure.
+	Since the DVD structure is 2052 bytes while the BD DI is 4100 bytes,
+	we can tell from the size whether we have been fooled. */
+
+   length = ab->buf[0]<<8 | ab->buf[1];
+   length += 2;
+   if(length != 4100) /* not a BD */
+   {  Verbose("# allocation length = %d != 4100 -> not a BD type medium.\n", length);
+      goto try_dvd;
+   }
+
+   Verbose("# -> Looks like a BD type medium.\n");
+   dh->profileDescr = "(BD)";
+   dh->shortProfile = "(BD)";
+   dh->mainType = DVD;
+   dh->subType = DVD;
+   FreeAlignedBuffer(ab);
+   return TRUE;
+
+   /*** If the medium is a DVD, the following query will succeed. */
+
+   /* Query length of returned data */
+try_dvd:
+   Verbose("# DVD: Trying READ DVD STRUCTURE.\n");
+
+   memset(cmd, 0, MAX_CDB_SIZE);
+   cmd[0] = 0xad;     /* READ DVD STRUCTURE */
+   cmd[6] = 0;        /* First layer */
+   cmd[7] = 0;        /* We want PHYSICAL info */
+   cmd[8] = 0;        /* Allocation length */
+   cmd[9] = 2;
+
+   /* Different drives react with different error codes on this request;
+      especially CDROMs seem to react very indeterministic here 
+      (okay, they obviously do not know about DVD cdbs).
+      So we do not look for specific error and regard any failure as a sign
+      that the medium is not a DVD. */
+
+   if(SendPacket(dh, cmd, 12, ab->buf, 2, sense, DATA_READ)<0)
+   {  
+      Verbose("# failed -> not a DVD type medium\n");
+      goto assume_cd;
+   }
+
+   Verbose("# -> Looks like a DVD type medium.\n");
+   dh->profileDescr = "(DVD)";
+   dh->shortProfile = "(DVD)";
+   dh->mainType = DVD;
+   dh->subType = DVD;
+   FreeAlignedBuffer(ab);
+   return TRUE;
+
+   /*** No need to investigate further; if it is not a CD some
+	subsequent tests will fail. */
+
+assume_cd:  
+   Verbose("# CD: It's either a CD or totally broken.\n");
+   dh->profileDescr = "(CD)";
+   dh->shortProfile = "(CD)";
+   dh->mainType = CD;
+   FreeAlignedBuffer(ab);
+   return TRUE;
+}
+
+/*
+ * Ask drive for active profile; this gives us a hint
  * about the kind of medium the drive believes to see
  */
 
@@ -160,6 +262,10 @@ static int get_configuration(DeviceHandle *dh)
    if(ret<0) 
    {  RememberSense(sense->sense_key, sense->asc, sense->ascq);
       Verbose("# -> failure\n");
+      FreeAlignedBuffer(ab);
+      if(try_fallback_type_check(dh))
+	return 0;
+      return ret;
    }
 
    data_length = ab->buf[0]<<24 | ab->buf[1] | ab->buf[2] | ab->buf[3];
@@ -171,85 +277,104 @@ static int get_configuration(DeviceHandle *dh)
    switch(dh->profile)
    {  /* Note: subType handling is different for CD for historical reasons */
       case 0x08: dh->profileDescr = "CD-ROM";
+	         dh->shortProfile = "CD-ROM";
                  dh->mainType = CD;
 	         break;
       case 0x09: dh->profileDescr = "CD-R";
+	         dh->shortProfile = "CD-R";
                  dh->mainType = CD;
 	         break;
       case 0x0a: dh->profileDescr = "CD-RW";
+	         dh->shortProfile = "CD-RW";
                  dh->mainType = CD;
 		 dh->rewriteable = TRUE;
 	         break;
 
       case 0x10: dh->profileDescr = "DVD-ROM";
+	         dh->shortProfile = "DVD-ROM";
                  dh->mainType = DVD;
 		 dh->subType = DVD;
 	         break;
       case 0x11: dh->profileDescr = "DVD-R Sequential recording";
+	         dh->shortProfile = "DVD-R";
 	         dh->mainType = DVD; dh->isDash = TRUE;
 		 dh->subType = DVD_DASH_R;
 	         break;
       case 0x12: dh->profileDescr = "DVD-RAM";
+	         dh->shortProfile = "DVD-RAM";
                  dh->mainType = DVD;
 		 dh->rewriteable = TRUE;
 		 dh->subType = DVD_RAM;
 	         break;
       case 0x13: dh->profileDescr = "DVD-RW Restricted overwrite";
+	         dh->shortProfile = "DVD-RW";
 	         dh->mainType = DVD; dh->isDash = TRUE;
 		 dh->rewriteable = TRUE;
 		 dh->subType = DVD_DASH_RW;
 	         break;
       case 0x14: dh->profileDescr = "DVD-RW Sequential overwrite";
+	         dh->shortProfile = "DVD-RW";
 	         dh->mainType = DVD; dh->isDash = TRUE;
 		 dh->rewriteable = TRUE;
 		 dh->subType = DVD_DASH_RW;
 	         break;
       case 0x15: dh->profileDescr = "DVD-R DL Sequential recording";
+	         dh->shortProfile = "DVD-R DL";
 	         dh->mainType = DVD; dh->isDash = TRUE;
 		 dh->subType = DVD_DASH_R_DL;
 	         break;
       case 0x16: dh->profileDescr = "DVD-R DL Layer jump recording";
+	         dh->shortProfile = "DVD-R DL";
 	         dh->mainType = DVD; dh->isDash = TRUE;
 		 dh->subType = DVD_DASH_R_DL;
 	         break;
       case 0x17: dh->profileDescr = "DVD-RW DL";
+	         dh->shortProfile = "DVD-RW DL";
 	         dh->mainType = DVD; dh->isDash = TRUE;
 		 dh->rewriteable = TRUE;
 		 dh->subType = DVD_DASH_RW_DL;
 	         break;
 
       case 0x1a: dh->profileDescr = "DVD+RW";
+	         dh->shortProfile = "DVD+RW";
 	         dh->mainType = DVD; dh->isPlus = TRUE;
 		 dh->rewriteable = TRUE;
 		 dh->subType = DVD_PLUS_RW;
 	         break;
       case 0x1b: dh->profileDescr = "DVD+R";
+	         dh->shortProfile = "DVD+R";
 	         dh->mainType = DVD; dh->isPlus = TRUE;
 		 dh->subType = DVD_PLUS_R;
 	         break;
       case 0x2a: dh->profileDescr = "DVD+RW DL";
+	         dh->shortProfile = "DVD+RW DL";
 	         dh->mainType = DVD; dh->isPlus = TRUE;
 		 dh->rewriteable = TRUE;
 		 dh->subType = DVD_PLUS_RW_DL;
 	         break;
       case 0x2b: dh->profileDescr = "DVD+R DL";
+	         dh->shortProfile = "DVD+R DL";
 	         dh->mainType = DVD; dh->isPlus = TRUE;
 		 dh->subType = DVD_PLUS_R_DL;
 	         break;
 
       case 0x40: dh->profileDescr = "BD-ROM";
+	         dh->shortProfile = "BD-ROM";
 	         dh->mainType = BD;
 		 dh->subType = BD;
 	         break;
       case 0x41: dh->profileDescr = "BD-R Sequential recording mode";
+ 	         dh->shortProfile = "BD-R";
 	         dh->mainType = BD;
 		 dh->subType = BD_R;
 	         break;
       case 0x42: dh->profileDescr = "BD-R Random recording mode";
+ 	         dh->shortProfile = "BD-R";
 	         dh->mainType = BD;
 		 dh->subType = BD_R;
 	         break;
       case 0x43: dh->profileDescr = "BD-RE";
+ 	         dh->shortProfile = "BD-RE";
 	         dh->mainType = BD;
 		 dh->rewriteable = TRUE;
 		 dh->subType = BD_RE;
@@ -261,9 +386,43 @@ static int get_configuration(DeviceHandle *dh)
 	         break;
    }
 
+   Verbose("-> profile %x: %s\n", dh->profile, dh->profileDescr);
    return ret;
 }
 
+/*
+ * Incomplete/unfinalized media usually cause big trouble
+ * as the usual probes (reading the TOC etc. fail).
+ * However sometimes they are still readable sectorwise,
+ * so see if we can read the ISO root sector.
+ * If we can, close eyes and continue. 
+ */
+
+static int query_incomplete(DeviceHandle *dh, int probe_only)
+{  AlignedBuffer *ab = CreateAlignedBuffer(2048);
+   int status;
+
+   Verbose("#QUERY INCOMPLETE: probing ISO root sector\n");
+
+   status = dh->read(dh, ab->buf, 16, 1);
+   if(status)
+   {  Verbose("# Readiing the ISO root sector failed\n");
+      FreeAlignedBuffer(ab);
+      return FALSE;
+   }
+
+   if(!strncmp((char*)(ab->buf+1), "CD001", 5))
+   {  Verbose("#QUERY INCOMPLETE: sector 16 looks like an ISO root sector\n");
+      dh->typeDescr = g_strdup_printf(_("Incomplete %s"), dh->shortProfile);
+      FreeAlignedBuffer(ab);
+      return TRUE;
+   }
+   else Verbose("-> not an ISO root sector\n");
+
+   FreeAlignedBuffer(ab);
+   return FALSE;
+}
+
 /*
  * CD specific probing
  */
@@ -289,14 +448,15 @@ static int query_cd(DeviceHandle *dh, int probe_only)
 
    Verbose("#CD: querying size of READ TOC/PMA/ATIP (for TOC)\n");
    if(SendPacket(dh, cmd, 10, buf, 2, sense, DATA_READ)<0)
-   {  if(!probe_only)
-      {  FreeAlignedBuffer(ab);
+   {  
+      FreeAlignedBuffer(ab);
+      if(!probe_only)
          Stop(_("%s\nCould not query TOC length.\n"),
 	      GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
-      }
+
       /* Blank CDs have no TOC, so they fail here.
 	 Give back some meaningful medium description. */
-      dh->typeDescr = g_strdup_printf("%s (%s)", dh->profileDescr, _("blank"));
+      dh->typeDescr = g_strdup_printf("%s (%s)", dh->shortProfile, _("blank"));
       return FALSE;
    }
 
@@ -309,7 +469,8 @@ static int query_cd(DeviceHandle *dh, int probe_only)
 	 HexDump(buf, 1024, 16);
 
       FreeAlignedBuffer(ab);
-      Stop(_("TOC info too long (%d), probably multisession.\n"),length);
+      if(!probe_only)
+	 Stop(_("TOC info too long (%d), probably multisession.\n"),length);
       return FALSE;
    }
 
@@ -323,8 +484,9 @@ static int query_cd(DeviceHandle *dh, int probe_only)
    Verbose("#CD: querying real READ TOC/PMA/ATIP (for TOC)\n");
    if(SendPacket(dh, cmd, 10, buf, length, sense, DATA_READ)<0)
    {  FreeAlignedBuffer(ab);
-      Stop(_("%s\nCould not read TOC.\n"),
-	   GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
+      if(!probe_only)
+	 Stop(_("%s\nCould not read TOC.\n"),
+	      GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
       return FALSE;
    }
    if(Closure->verbose)
@@ -332,7 +494,7 @@ static int query_cd(DeviceHandle *dh, int probe_only)
 
    control = buf[5];
    Verbose("#CD: control is 0x%x\n", control);
-   
+
    /*** Do the READ TOC again with format 2 to fetch the full toc
         as we want the disc type info also. 
 	We do not use the CONTROL data included here as it turned
@@ -349,8 +511,9 @@ static int query_cd(DeviceHandle *dh, int probe_only)
    Verbose("#CD: querying size of READ TOC/PMA/ATIP (for full TOC)\n");
    if(SendPacket(dh, cmd, 10, buf, 2, sense, DATA_READ)<0)
    {  FreeAlignedBuffer(ab);
-      Stop(_("%s\nCould not query full TOC length.\n"),
-	   GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
+      if(!probe_only)
+	 Stop(_("%s\nCould not query full TOC length.\n"),
+	      GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
       return FALSE;
    }
 
@@ -360,15 +523,19 @@ static int query_cd(DeviceHandle *dh, int probe_only)
 
    if(length < 15)
    {  FreeAlignedBuffer(ab);
-      Stop(_("TOC info too short, length %d.\n"),length);
+      if(!probe_only)
+	 Stop(_("TOC info too short, length %d.\n"),length);
       return FALSE;
    }
    if(length>1024) /* don't let the drive hack us using a buffer overflow ;-) */
    {  FreeAlignedBuffer(ab);
-      Stop(_("TOC info too long (%d), probably multisession.\n"),length);
+      if(!probe_only)
+	 Stop(_("TOC info too long (%d), probably multisession.\n"),length);
       return FALSE;
    }
 
+   length = 16;  /* Works around Windows (and possibly other OS) driver issues */
+
    memset(cmd, 0, MAX_CDB_SIZE);
    cmd[0] = 0x43;  /* READ TOC/PMA/ATIP */
    cmd[1] = 0x02;  /* TIME bit required for this format */
@@ -380,8 +547,9 @@ static int query_cd(DeviceHandle *dh, int probe_only)
    Verbose("#CD: querying real READ TOC/PMA/ATIP (for full TOC)\n");
    if(SendPacket(dh, cmd, 10, buf, length, sense, DATA_READ)<0)
    {  FreeAlignedBuffer(ab);
-      Stop(_("%s\nCould not read full TOC.\n"),
-	   GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
+      if(!probe_only)
+	 Stop(_("%s\nCould not read full TOC.\n"),
+	      GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
       return FALSE;
    }
    if(Closure->verbose)
@@ -396,7 +564,8 @@ static int query_cd(DeviceHandle *dh, int probe_only)
 	 if(i==3 || (i-3)%11 == 0) PrintLog("\n");
       }
       FreeAlignedBuffer(ab);
-      Stop(_("Consider sending a bug report.\n"));
+      if(!probe_only)
+	 Stop(_("Consider sending a bug report.\n"));
       return FALSE;
    }
 
@@ -459,7 +628,8 @@ static int query_dvd(DeviceHandle *dh, int probe_only)
 
    if(length>4096) /* don't let the drive hack us using a buffer overflow ;-) */
    {  FreeAlignedBuffer(ab);
-      Stop(_("Could not query dvd physical structure - implausible packet length %d\n"),length);
+      if(!probe_only)
+	 Stop(_("Could not query dvd physical structure - implausible packet length %d\n"),length);
       return FALSE;
    }
    Verbose("#DVD: size returned is %d\n", length);
@@ -476,8 +646,9 @@ static int query_dvd(DeviceHandle *dh, int probe_only)
    Verbose("#DVD: trying READ DVD for real PHYSICAL info\n");
    if(SendPacket(dh, cmd, 12, buf, length, sense, DATA_READ)<0)
    {  FreeAlignedBuffer(ab);
-      Stop(_("%s\nCould not query physical dvd structure.\n"),
-	   GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
+      if(!probe_only)
+	 Stop(_("%s\nCould not query physical dvd structure.\n"),
+	      GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
       return FALSE;
    }
    if(Closure->verbose)
@@ -782,15 +953,6 @@ static int query_bd(DeviceHandle *dh, int probe_only)
    length += 2;
    Verbose("#BD: disc structure query succeeded, length %d bytes\n", length);
 
-#if 0  /* no longer needed here */
-   /*** Some DVD drives ignore the media type 0x01 and return the dvd structure.
-	Since the DVD structure is 2052 bytes while the BD DI is 4100 bytes,
-	we can tell from the size whether we have been fooled. */
-
-   if(length != 4100) /* not a BD */
-      goto try_dvd;
-#endif
-
    /* Do the real query */
 
    memset(cmd, 0, MAX_CDB_SIZE);
@@ -804,8 +966,9 @@ static int query_bd(DeviceHandle *dh, int probe_only)
    Verbose("#BD: trying READ DISC STRUCTURE for real query\n");
    if(SendPacket(dh, cmd, 12, buf, length, sense, DATA_READ)<0)
    {  FreeAlignedBuffer(ab);
-      Stop(_("%s\nCould not query BD disc structure.\n"),
-	   GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
+      if(!probe_only)
+	 Stop(_("%s\nCould not query BD disc structure.\n"),
+	      GetSenseString(sense->sense_key, sense->asc, sense->ascq, TRUE));
       return FALSE;
    }
 
@@ -864,7 +1027,12 @@ static int query_bd(DeviceHandle *dh, int probe_only)
  */
 
 static int query_type(DeviceHandle *dh, int probe_only)
-{  int status;
+{  AlignedBuffer *ab;
+   unsigned char *buf;
+   unsigned char cmd[MAX_CDB_SIZE];
+   Sense *sense = &dh->sense;
+   unsigned int length;
+   int status;
 
    /*** See which profile the drive selected.
 	This should at least give us a hint to decide 
@@ -876,59 +1044,10 @@ static int query_type(DeviceHandle *dh, int probe_only)
    if(status)  /* something went terribly wrong */
      return FALSE;
 
-   /*** Some defaults */
-
-   strcpy(dh->manuID, "-");
-   dh->bookDescr = "-";
-   
-   /*** Main type has been decided now */
-
-   switch(dh->mainType)
-   {  case CD:
-        dh->read        = read_cd_sector;
-	dh->readRaw     = read_raw_cd_sector;
-	dh->singleRate  = 150.0;
-	dh->maxRate     = 52;
-	dh->clusterSize = 16;  /* really 1, but this is faster */ 
-	return query_cd(dh, probe_only);
-
-      case DVD:
-	dh->read        = read_dvd_sector;
-	dh->singleRate  = 1352.54;
-	dh->maxRate     = 17;
-	dh->clusterSize = 16;
-	return query_dvd(dh, probe_only);
-
-      case BD:
-	dh->read        = read_dvd_sector;
-	dh->singleRate  = 36000.0/8.0;  /* 1x = 36 kbit */
-	dh->maxRate     = 9;
-	dh->clusterSize = 32;
-	return query_bd(dh, probe_only);
-
-      default:  /* maybe HD DVD or sth else we do not support */
-	return FALSE;
-   }
-
-   return FALSE; /* unreachable */
-}
-
-/*
- * Find out whether the disc is blank,
- * and the blank disc capacity.
- */
-
-static int query_blank(DeviceHandle *dh)
-{  AlignedBuffer *ab = CreateAlignedBuffer(4096);
-   unsigned char *buf = ab->buf;
-   unsigned char cmd[MAX_CDB_SIZE];
-   Sense *sense = &dh->sense;
-   unsigned int length;
-
-   Verbose("# *** QueryBlank(%s) ***\n", dh->devinfo);
-
-   /* Get disc information to learn the disc state (blank, finalized etc.) */
+   /*** Get disc information to learn the disc state (blank, finalized etc.) */
 
+   ab = CreateAlignedBuffer(4096);
+   buf = ab->buf;
    memset(cmd, 0, MAX_CDB_SIZE);
    cmd[0] = 0x51;     /* READ DISC INFORMATION */
    cmd[1] = 0;        /* standard disc info */
@@ -950,21 +1069,86 @@ static int query_blank(DeviceHandle *dh)
       Verbose("# trying READ DISC INFORMATION for real info\n");
       if(SendPacket(dh, cmd, 10, buf, length, sense, DATA_READ) == 0)
       {  dh->discStatus = buf[2];
+	 status = (dh->discStatus>>2)&3;
+	 if(status == 1 || status == 2) /* incomplete or damaged */
+	    dh->incomplete = TRUE;
+	 if(dh->mainType == CD)
+	    switch(buf[8])
+	    {  case    0: dh->subType = DATA1; break;
+	       case 0x20: dh->subType  = XA21; break; 
+	    }
+
 	 if(Closure->verbose)
 	   HexDump(buf, length, 16);
- 	 Verbose("# status is %x\n", dh->discStatus);
-      }
-      else 
-      {  FreeAlignedBuffer(ab);
-	 return FALSE;
+ 	 Verbose("# status is %x%s disc type %x\n", 
+		 dh->discStatus, dh->incomplete ? " (incomplete!),":",", buf[8]);
       }
+      else Verbose("# READ DISC INFORMATION failed\n");
    }
-   else 
-   {  Verbose("# READ DISC INFORMATION failed\n");
-      FreeAlignedBuffer(ab);
-      return FALSE;
+   else Verbose("# Getting size of READ DISC INFORMATION failed\n");
+   FreeAlignedBuffer(ab);
+
+   /*** Some defaults */
+
+   strcpy(dh->manuID, "-");
+   dh->bookDescr = "-";
+   
+   /*** Main type has been decided now */
+
+   switch(dh->mainType)
+   {  case CD:
+        dh->read        = read_cd_sector;
+	dh->readRaw     = read_raw_cd_sector;
+	dh->singleRate  = 150.0;
+	dh->maxRate     = 52;
+	dh->clusterSize = 16;  /* really 1, but this is faster */ 
+	return dh->incomplete ? query_incomplete(dh, probe_only) : query_cd(dh, probe_only);
+
+      case DVD:
+	dh->read        = read_dvd_sector;
+	dh->singleRate  = 1352.54;
+	dh->maxRate     = 17;
+	dh->clusterSize = 16;
+	if(!dh->incomplete) return query_dvd(dh, probe_only);
+	else
+	{  if(query_dvd(dh, TRUE)) return TRUE;
+	   return query_incomplete(dh, probe_only);
+	}
+	break;
+
+      case BD:
+	dh->read        = read_dvd_sector;
+	dh->singleRate  = 36000.0/8.0;  /* 1x = 36 kbit */
+	dh->maxRate     = 9;
+	dh->clusterSize = 32;
+	if(!dh->incomplete) return query_bd(dh, probe_only);
+	else
+	{  if(query_bd(dh, TRUE)) return TRUE;
+	   return query_incomplete(dh, probe_only);
+	}
+	break;
+
+      default:  /* maybe HD DVD or sth else we do not support */
+	return FALSE;
    }
 
+   return FALSE; /* unreachable */
+}
+
+/*
+ * Find out whether the disc is blank,
+ * and the blank disc capacity.
+ */
+
+static int query_blank(DeviceHandle *dh)
+{  AlignedBuffer *ab = CreateAlignedBuffer(4096);
+   unsigned char *buf = ab->buf;
+   unsigned char cmd[MAX_CDB_SIZE];
+   Sense *sense = &dh->sense;
+   unsigned int length;
+
+   Verbose("# *** QueryBlank(%s) ***\n", dh->devinfo);
+
    /*** Determine blank size for CD media ***/
 
    if(dh->mainType == CD)
@@ -1066,6 +1250,7 @@ static int query_blank(DeviceHandle *dh)
 	 case DVD_PLUS_RW:
 	 case DVD_DASH_RW_DL:
 	 case DVD_PLUS_RW_DL:
+	 case DVD_RAM:
          {  int cmc_size;
 	    int n_lists;
 	    int i,idx;
@@ -1441,6 +1626,11 @@ static int query_copyright(DeviceHandle *dh)
    unsigned char cmd[MAX_CDB_SIZE];
    unsigned char result;
    unsigned int length;
+   
+   /* The following test does neither apply nor is necessary
+      for incomplete (damaged) discs. */ 
+
+   if(dh->incomplete) return FALSE;
 
    /* Query length of returned data */
 
@@ -1485,7 +1675,7 @@ static int query_copyright(DeviceHandle *dh)
 
    result = buf[4] & 0x11;
    FreeAlignedBuffer(ab);
-   
+
    return result;
 }
 
@@ -1571,6 +1761,7 @@ static unsigned int query_size(DeviceHandle *dh)
 
    /*** Query size by doing READ CAPACITY */
 
+   Verbose("# *** query_size(%s) ***\n", dh->devinfo);
    memset(cmd, 0, MAX_CDB_SIZE);
    cmd[0] = 0x25;  /* READ CAPACITY */
 
@@ -1884,7 +2075,7 @@ void LoadMedium(DeviceHandle *dh, int load)
    cmd[0] = 0x1b;                /* START STOP */
    cmd[4] = load ? 0x03 : 0x02;  /* LOEJ=1; START=load/eject */
 
-   if(SendPacket(dh, cmd, 12, NULL, 0, &sense, DATA_READ)<0
+   if(SendPacket(dh, cmd, 6, NULL, 0, &sense, DATA_READ)<0
       && (sense.asc != 0x53 || sense.ascq != 0x02))
    {
       PrintLog(_("%s\nCould not load/unload the medium.\n"), 
@@ -1902,7 +2093,7 @@ void LoadMedium(DeviceHandle *dh, int load)
    memset(cmd, 0, MAX_CDB_SIZE);
    cmd[0] = 0x1e;                /* PREVENT ALLOW MEDIUM REMOVAL */
 
-   if(SendPacket(dh, cmd, 12, NULL, 0, &sense, DATA_READ)<0)
+   if(SendPacket(dh, cmd, 6, NULL, 0, &sense, DATA_READ)<0)
       PrintLog(_("%s\nCould not unlock the medium.\n"), 
 	       GetSenseString(sense.sense_key, sense.asc, sense.ascq, TRUE));
 
@@ -1912,7 +2103,7 @@ void LoadMedium(DeviceHandle *dh, int load)
    cmd[0] = 0x1b;                /* START STOP */
    cmd[4] = 0x02;                /* LOEJ=1; START=eject */
 
-   if(SendPacket(dh, cmd, 12, NULL, 0, &sense, DATA_READ)<0)
+   if(SendPacket(dh, cmd, 6, NULL, 0, &sense, DATA_READ)<0)
       PrintLog(_("%s\nCould not load/unload the medium.\n"), 
 	       GetSenseString(sense.sense_key, sense.asc, sense.ascq, TRUE));
 }
@@ -1930,7 +2121,7 @@ int TestUnitReady(DeviceHandle *dh)
    memset(cmd, 0, MAX_CDB_SIZE);
    cmd[0] = 0x00;     /* TEST UNIT READY */
 
-   if(SendPacket(dh, cmd, 12, NULL, 0, &dh->sense, DATA_READ) != -1)
+   if(SendPacket(dh, cmd, 6, NULL, 0, &dh->sense, DATA_READ) != -1)
       return TRUE;
 
    /*** If no medium present, try closing the tray. */
@@ -1943,10 +2134,12 @@ int TestUnitReady(DeviceHandle *dh)
         becoming ready */
 
    for(i=0; i<10; i++)
-   {  memset(cmd, 0, MAX_CDB_SIZE);
+   {  int continue_waiting = FALSE;
+
+      memset(cmd, 0, MAX_CDB_SIZE);
       cmd[0] = 0x00;     /* TEST UNIT READY */
 
-      if(SendPacket(dh, cmd, 12, NULL, 0, &dh->sense, DATA_READ) != -1)
+      if(SendPacket(dh, cmd, 6, NULL, 0, &dh->sense, DATA_READ) != -1)
       {  if(Closure->guiMode)
 	    SetLabelText(Closure->status, "");
 	 return TRUE;
@@ -1956,16 +2149,24 @@ int TestUnitReady(DeviceHandle *dh)
 	 if(   (dh->sense.asc == 0x04 && dh->sense.ascq == 0x01) /* but in process of becoming so */
 	     || dh->sense.asc == 0x3a    /* flawed firmware workaround */
 	   )
-	 {  PrintCLIorLabel(Closure->status,
-			    _("Waiting 10 seconds for drive: %d\n"),9-i);
+	    continue_waiting = TRUE;
 
-	    if(Closure->stopActions)
-	       return FALSE;
+      if(dh->sense.sense_key == 6) /* Unit attention */
+	 if(dh->sense.asc == 0x28 && dh->sense.ascq == 0x00)
+	    continue_waiting = TRUE;
 
-	    g_usleep(G_USEC_PER_SEC);
-	    continue;
-	 }
-      printf("fell out %x %x %x\n",dh->sense.sense_key,dh->sense.asc,dh->sense.ascq);
+      if(continue_waiting)
+      {  PrintCLIorLabel(Closure->status,
+			 _("Waiting 10 seconds for drive: %d\n"),9-i);
+
+	 if(Closure->stopActions)
+	    return FALSE;
+
+	 g_usleep(G_USEC_PER_SEC);
+	 continue;
+      }
+
+      Verbose("fell out waiting for drive %x %x %x\n",dh->sense.sense_key,dh->sense.asc,dh->sense.ascq);
       break;  /* Something is wrong with the drive */
    }
 
@@ -2035,6 +2236,12 @@ static int read_cd_sector(DeviceHandle *dh, unsigned char *buf, int lba, int nse
 
    ret = SendPacket(dh, cmd, 12, buf, 2048*nsectors, sense, DATA_READ);
 
+#if 0
+   if(lba==16000)  /* fixme */
+   {  buf[240]^=255;
+   }
+#endif
+
    if(ret<0) RememberSense(sense->sense_key, sense->asc, sense->ascq);
    return ret;
 }
@@ -2066,7 +2273,6 @@ static int read_raw_cd_sector(DeviceHandle *dh, unsigned char *outbuf, int lba,
    /*** Perform the raw read */
 
    c2bit = dh->canC2Scan ? 0x02 : 0;
-
    memset(cdb, 0, MAX_CDB_SIZE);
    cdb[0]  = 0xbe;         /* READ CD */
    switch(dh->subType)     /* Expected sector type */
@@ -2097,14 +2303,16 @@ static int read_raw_cd_sector(DeviceHandle *dh, unsigned char *outbuf, int lba,
    /* initialize sectors with our marker */
 
    for(i=0, s=0; i<nsectors; i++, s+=rb->sampleSize)
-     memcpy(rb->workBuf->buf+s, Closure->deadSector, 2048);
+      CreateMissingSector(rb->workBuf->buf+s, lba+i, NULL, 0,
+			  "read_raw_cd_sector() dummy sector");
 
    ret = SendPacket(dh, cdb, 12, rb->workBuf->buf, nsectors*rb->sampleSize, sense, DATA_READ);
    RememberSense(sense->sense_key, sense->asc, sense->ascq);
 
 #if 0
-   if(lba==160)  /* fixme */
-   rb->workBuf->buf[230]^=255;
+   if(lba==16000)  /* fixme */
+   {  rb->workBuf->buf[0*rb->sampleSize+230]^=255;
+   }
 #endif
 
    /* count c2 error bits */
@@ -2140,11 +2348,11 @@ static int read_raw_cd_sector(DeviceHandle *dh, unsigned char *outbuf, int lba,
    /* See if drive returned some data at all. */
 
    for(i=0, s=0; i<nsectors; i++, s+=rb->sampleSize)
-     if(!memcmp(rb->workBuf->buf+s, Closure->deadSector, 2048))
-     {  if(dh->canReadDefective)
- 	  RememberSense(3, 255, 0); /* report that nothing came back */
-        return -1;
-     }
+      if(CheckForMissingSector(rb->workBuf->buf+s, lba+i, NULL, 0) != SECTOR_PRESENT)
+      {  if(dh->canReadDefective)
+	    RememberSense(3, 255, 0); /* report that nothing came back */
+	 return -1;
+      }
 
    /* Verify data and copy it back */
 
@@ -2178,8 +2386,7 @@ int ReadSectors(DeviceHandle *dh, unsigned char *buf, gint64 s, int nsectors)
    {  gint64 i,idx;
 
      for(idx=s,i=0; i<nsectors; idx++,i++)
-	//       if(GetBit(dh->defects, idx))
-	if(GetBit(dh->defects, idx) && !((Random() & 15)))
+       if(GetBit(dh->defects, idx) && !((Random() & 15)))
        {  dh->sense.sense_key = 3;
 	  dh->sense.asc       = 255;
 	  dh->sense.ascq      = 255;
@@ -2188,6 +2395,18 @@ int ReadSectors(DeviceHandle *dh, unsigned char *buf, gint64 s, int nsectors)
        }
    }
 
+#if 0
+   if(   (s == 331600 && nsectors > 16)
+	 || s >331605)
+   {  dh->sense.sense_key = 3;
+      dh->sense.asc       = 255;
+      dh->sense.ascq      = 255;
+      RememberSense(dh->sense.sense_key, dh->sense.asc, dh->sense.ascq);
+	  
+      return TRUE;
+   }
+#endif
+
    /* Reset raw reading buffer (if there is one) */
 
    if(Closure->readRaw && dh->rawBuffer)
@@ -2205,11 +2424,6 @@ int ReadSectors(DeviceHandle *dh, unsigned char *buf, gint64 s, int nsectors)
 	   status = dh->readRaw(dh, buf, s, nsectors);
       else status = dh->read(dh, buf, s, nsectors);
 
-#if 0
-   if(s==160 || s>=144624)  /* fixme */
-      buf[230]^=255;
-#endif
-
       if(Closure->readRaw && dh->rawBuffer)
 	recommended_attempts = dh->rawBuffer->recommendedAttempts;
 
@@ -2222,7 +2436,9 @@ int ReadSectors(DeviceHandle *dh, unsigned char *buf, gint64 s, int nsectors)
 	 /* Do not attempt multiple re-reads if nsectors > 1 and sectorSkip == 0
 	    as these will be re-read with nsectors==1 anyways. */
 
-	 if(dh->canReadDefective && nsectors > 1 && Closure->sectorSkip == 0)
+//       Why only apply this shortcut to raw reading?
+//	 if(dh->canReadDefective && nsectors > 1 && Closure->sectorSkip == 0)
+	 if(nsectors > 1 && Closure->sectorSkip == 0)
 	 {  PrintCLIorLabel(Closure->status,
 			    _("Sectors %lld - %lld: %s\n"),
 			    s, s+nsectors-1, GetLastSenseString(FALSE));
@@ -2339,9 +2555,13 @@ DeviceHandle* OpenAndQueryDevice(char *device)
    /* Query the type and fail immediately if incompatible medium is found
       so that the later tests are not derailed by the wrong medium type */
 
-   query_type(dh, 0);
+   if(!query_type(dh, 0))
+   {  Stop(_("Drive failed to report media type."));
+      return NULL;
+   }
+     
    Verbose("# query_type() returned.\n");
-   
+
    if(dh->subType == UNSUPPORTED)
    {  char *td = alloca(strlen(dh->typeDescr)+1);
 
@@ -2464,6 +2684,7 @@ DeviceHandle* QueryMediumInfo(char *device)
 
    /* Open the device. */
 
+   Verbose("# *** QueryMediumInfo(%s) ***\n", device);
    dh = OpenDevice(device);
    InquireDevice(dh, 0);
 
@@ -2473,6 +2694,7 @@ DeviceHandle* QueryMediumInfo(char *device)
 	   Stop(_("Device %s: no medium present\n"), device);
       else Stop(_("Device %s does not become ready:\n%s\n\n"), device,
 		GetSenseString(dh->sense.sense_key, dh->sense.asc, dh->sense.ascq, FALSE));
+      return NULL;
    }
 
    /* General type detection */
@@ -2508,6 +2730,7 @@ int GetMediumFingerprint(DeviceHandle *dh, guint8 *fp_out, gint64 sector)
       {  case 0:    /* not read */
 	    break;
 	 case 1:    /* unreadable */
+	    memset(fp_out, 0, 16);
 	    return FALSE;
 	 case 2:    /* already cached */
 	    memcpy(fp_out, dh->mediumFP, 16);
