@@ -1,5 +1,5 @@
 /*  dvdisaster: Additional error correction for optical media.
- *  Copyright (C) 2004-2008 Carsten Gnoerlich.
+ *  Copyright (C) 2004-2009 Carsten Gnoerlich.
  *  Project home page: http://www.dvdisaster.com
  *  Email: carsten@dvdisaster.com  -or-  cgnoerlich@fsfe.org
  *
@@ -22,7 +22,7 @@
 #include "dvdisaster.h"
 
 //#define LOCAL_ONLY 1
-#define PRINT_STEPS 1
+//#define PRINT_STEPS 1
 
 #define VERBOSE 1
 #ifdef VERBOSE
@@ -1647,7 +1647,8 @@ int SmartLEC(RawBuffer *rb)
    printf("%s\n", shc->msg);
 #endif
 
-   memcpy(prev_state, Closure->deadSector, 2048);
+   CreateMissingSector(prev_state, rb->lba, NULL, 0,
+		       "SmartLEC() dummy sector");
    while(TRUE)
    {  smart_lec_iteration(shc, NULL);
 #ifdef PRINT_STEPS
